namespace com.progsbase.no.inductive.idea10.programs.JSON_v0_3_9{

using static System.Math;

public class ElementArrayReference{
	public Element [] array;
}
public class Element{
	public ElementType type;
	public StringElementMap objectx;
	public Element [] array;
	public char [] stringx;
	public double number;
	public bool booleanValue;
}
public class ElementReference{
	public Element element;
}
public class ElementType{
	public char [] name;
}
public class Token{
	public TokenType type;
	public char [] value;
}
public class TokenReference{
	public Token token;
}
public class TokenType{
	public char [] name;
}
public class TokenArrayReference{
	public Token [] array;
}
public class StringElementMap{
	public StringArrayReference stringListRef;
	public ElementArrayReference elementListRef;
}
public class BooleanArrayReference{
	public bool [] booleanArray;
}
public class BooleanReference{
	public bool booleanValue;
}
public class CharacterReference{
	public char characterValue;
}
public class NumberArrayReference{
	public double [] numberArray;
}
public class NumberReference{
	public double numberValue;
}
public class StringArrayReference{
	public StringReference [] stringArray;
}
public class StringReference{
	public char [] stringx;
}
public class JSON{
	public static bool IsValidJSON(char [] json, StringArrayReference errorMessage){
		bool success;
		ElementReference elementReference;

		elementReference = new ElementReference();

		success = ReadJSON(json, elementReference, errorMessage);

		if(success){
			DeleteElement(elementReference.element);
		}

		return success;
	}


	public static bool JSONTokenize(char [] stringx, TokenArrayReference tokenArrayReference, StringArrayReference errorMessages){
		NumberReference count;
		bool success;

		count = CreateNumberReference(0d);
		success = JSONTokenizeWithCountOption(stringx, tokenArrayReference, count, false, errorMessages);

		if(success){
			tokenArrayReference.array = new Token [(int)(count.numberValue)];
			JSONTokenizeWithCountOption(stringx, tokenArrayReference, count, true, errorMessages);
		}

		return success;
	}


	public static bool JSONTokenizeWithCountOption(char [] json, TokenArrayReference tokenArrayReference, NumberReference countReference, bool add, StringArrayReference errorMessages){
		Token [] tokens;
		double i, t;
		char c;
		Token numberToken;
		char [] str;
		StringReference stringReference;
		TokenReference tokenReference;
		NumberReference stringLength;
		bool success;

		success = true;

		tokenReference = new TokenReference();
		countReference.numberValue = 0d;

		tokens = tokenArrayReference.array;
		stringLength = new NumberReference();
		t = 0d;

		for(i = 0d; i < json.Length && success; ){
			c = json[(int)(i)];

			if(c == '{'){
				if(add){
					tokens[(int)(t)] = CreateToken(GetTokenType("openCurly".ToCharArray()));
					t = t + 1d;
				}else{
					countReference.numberValue = countReference.numberValue + 1d;
				}
				i = i + 1d;
			}else if(c == '}'){
				if(add){
					tokens[(int)(t)] = CreateToken(GetTokenType("closeCurly".ToCharArray()));
					t = t + 1d;
				}else{
					countReference.numberValue = countReference.numberValue + 1d;
				}
				i = i + 1d;
			}else if(c == '['){
				if(add){
					tokens[(int)(t)] = CreateToken(GetTokenType("openSquare".ToCharArray()));
					t = t + 1d;
				}else{
					countReference.numberValue = countReference.numberValue + 1d;
				}
				i = i + 1d;
			}else if(c == ']'){
				if(add){
					tokens[(int)(t)] = CreateToken(GetTokenType("closeSquare".ToCharArray()));
					t = t + 1d;
				}else{
					countReference.numberValue = countReference.numberValue + 1d;
				}
				i = i + 1d;
			}else if(c == ':'){
				if(add){
					tokens[(int)(t)] = CreateToken(GetTokenType("colon".ToCharArray()));
					t = t + 1d;
				}else{
					countReference.numberValue = countReference.numberValue + 1d;
				}
				i = i + 1d;
			}else if(c == ','){
				if(add){
					tokens[(int)(t)] = CreateToken(GetTokenType("comma".ToCharArray()));
					t = t + 1d;
				}else{
					countReference.numberValue = countReference.numberValue + 1d;
				}
				i = i + 1d;
			}else if(c == 'f'){
				success = GetJSONPrimitiveName(json, i, errorMessages, "false".ToCharArray(), tokenReference);
				if(success){
					if(add){
						tokens[(int)(t)] = tokenReference.token;
						t = t + 1d;
					}else{
						countReference.numberValue = countReference.numberValue + 1d;
					}
					i = i + "false".ToCharArray().Length;
				}
			}else if(c == 't'){
				success = GetJSONPrimitiveName(json, i, errorMessages, "true".ToCharArray(), tokenReference);
				if(success){
					if(add){
						tokens[(int)(t)] = tokenReference.token;
						t = t + 1d;
					}else{
						countReference.numberValue = countReference.numberValue + 1d;
					}
					i = i + "true".ToCharArray().Length;
				}
			}else if(c == 'n'){
				success = GetJSONPrimitiveName(json, i, errorMessages, "null".ToCharArray(), tokenReference);
				if(success){
					if(add){
						tokens[(int)(t)] = tokenReference.token;
						t = t + 1d;
					}else{
						countReference.numberValue = countReference.numberValue + 1d;
					}
					i = i + "null".ToCharArray().Length;
				}
			}else if(c == ' ' || c == '\n' || c == '\t' || c == '\r'){
				/* Skip.*/
				i = i + 1d;
			}else if(c == '\"'){
				success = GetJSONString(json, i, tokenReference, stringLength, errorMessages);
				if(success){
					i = i + stringLength.numberValue;
					if(add){
						tokens[(int)(t)] = tokenReference.token;
						t = t + 1d;
					}else{
						countReference.numberValue = countReference.numberValue + 1d;
					}
				}
			}else if(IsJSONNumberCharacter(c)){
				success = GetJSONNumberToken(json, i, tokenReference, errorMessages);
				if(success){
					numberToken = tokenReference.token;
					i = i + numberToken.value.Length;
					if(add){
						tokens[(int)(t)] = numberToken;
						t = t + 1d;
					}else{
						countReference.numberValue = countReference.numberValue + 1d;
					}
				}
			}else{
				str = strConcatenateCharacter("Invalid start of Token: ".ToCharArray(), c);
				stringReference = CreateStringReference(str);
				AddStringRef(errorMessages, stringReference);
				i = i + 1d;
				success = false;
			}
		}

		if(success){
			if(add){
				tokens[(int)(t)] = CreateToken(GetTokenType("end".ToCharArray()));
				t = t + 1d;
			}else{
				countReference.numberValue = countReference.numberValue + 1d;
			}
			tokenArrayReference.array = tokens;
		}

		return success;
	}


	public static bool GetJSONNumberToken(char [] json, double start, TokenReference tokenReference, StringArrayReference errorMessages){
		char c;
		double end, i;
		bool done, success;
		char [] numberString;

		end = json.Length;
		done = false;

		for(i = start; i < json.Length && !done; i = i + 1d){
			c = json[(int)(i)];
			if(!IsJSONNumberCharacter(c)){
				done = true;
				end = i;
			}
		}

		numberString = strSubstring(json, start, end);

		success = IsValidJSONNumber(numberString, errorMessages);

		tokenReference.token = CreateNumberToken(numberString);

		return success;
	}


	public static bool IsValidJSONNumber(char [] n, StringArrayReference errorMessages){
		bool success;
		double i;

		i = 0d;

		/* JSON allows an optional negative sign.*/
		if(n[(int)(i)] == '-'){
			i = i + 1d;
		}

		if(i < n.Length){
			success = IsValidJSONNumberAfterSign(n, i, errorMessages);
		}else{
			success = false;
			AddStringRef(errorMessages, CreateStringReference("Number must contain at least one digit.".ToCharArray()));
		}

		return success;
	}


	public static bool IsValidJSONNumberAfterSign(char [] n, double i, StringArrayReference errorMessages){
		bool success;

		if(charIsNumber(n[(int)(i)])){
			/* 0 first means only 0.*/
			if(n[(int)(i)] == '0'){
				i = i + 1d;
			}else{
				/* 1-9 first, read following digits.*/
				i = IsValidJSONNumberAdvancePastDigits(n, i);
			}

			if(i < n.Length){
				success = IsValidJSONNumberFromDotOrExponent(n, i, errorMessages);
			}else{
				/* If integer, we are done now.*/
				success = true;
			}
		}else{
			success = false;
			AddStringRef(errorMessages, CreateStringReference("A number must start with 0-9 (after the optional sign).".ToCharArray()));
		}

		return success;
	}


	public static double IsValidJSONNumberAdvancePastDigits(char [] n, double i){
		bool done;

		i = i + 1d;
		done = false;
		for(; i < n.Length && !done; ){
			if(charIsNumber(n[(int)(i)])){
				i = i + 1d;
			}else{
				done = true;
			}
		}

		return i;
	}


	public static bool IsValidJSONNumberFromDotOrExponent(char [] n, double i, StringArrayReference errorMessages){
		bool wasDotAndOrE, success;

		wasDotAndOrE = false;
		success = true;

		if(n[(int)(i)] == '.'){
			i = i + 1d;
			wasDotAndOrE = true;

			if(i < n.Length){
				if(charIsNumber(n[(int)(i)])){
					/* Read digits following decimal point.*/
					i = IsValidJSONNumberAdvancePastDigits(n, i);

					if(i == n.Length){
						/* If non-scientific decimal number, we are done.*/
						success = true;
					}
				}else{
					success = false;
					AddStringRef(errorMessages, CreateStringReference("There must be numbers after the decimal point.".ToCharArray()));
				}
			}else{
				success = false;
				AddStringRef(errorMessages, CreateStringReference("There must be numbers after the decimal point.".ToCharArray()));
			}
		}

		if(i < n.Length && success){
			if(n[(int)(i)] == 'e' || n[(int)(i)] == 'E'){
				wasDotAndOrE = true;
				success = IsValidJSONNumberFromExponent(n, i, errorMessages);
			}else{
				success = false;
				AddStringRef(errorMessages, CreateStringReference("Expected e or E.".ToCharArray()));
			}
		}else if(i == n.Length && success){
			/* If number with decimal point.*/
			success = true;
		}else{
			success = false;
			AddStringRef(errorMessages, CreateStringReference("There must be numbers after the decimal point.".ToCharArray()));
		}

		if(wasDotAndOrE){
		}else{
			success = false;
			AddStringRef(errorMessages, CreateStringReference("Exprected decimal point or e or E.".ToCharArray()));
		}

		return success;
	}


	public static bool IsValidJSONNumberFromExponent(char [] n, double i, StringArrayReference errorMessages){
		bool success;

		i = i + 1d;

		if(i < n.Length){
			/* The exponent sign can either + or -,*/
			if(n[(int)(i)] == '+' || n[(int)(i)] == '-'){
				i = i + 1d;
			}

			if(i < n.Length){
				if(charIsNumber(n[(int)(i)])){
					/* Read digits following decimal point.*/
					i = IsValidJSONNumberAdvancePastDigits(n, i);

					if(i == n.Length){
						/* We found scientific number.*/
						success = true;
					}else{
						success = false;
						AddStringRef(errorMessages, CreateStringReference("There was characters following the exponent.".ToCharArray()));
					}
				}else{
					success = false;
					AddStringRef(errorMessages, CreateStringReference("There must be a digit following the optional exponent sign.".ToCharArray()));
				}
			}else{
				success = false;
				AddStringRef(errorMessages, CreateStringReference("There must be a digit following optional the exponent sign.".ToCharArray()));
			}
		}else{
			success = false;
			AddStringRef(errorMessages, CreateStringReference("There must be a sign or a digit following e or E.".ToCharArray()));
		}

		return success;
	}


	public static bool IsJSONNumberCharacter(char c){
		char [] numericCharacters;
		bool found;
		double i;

		numericCharacters = "0123456789.-+eE".ToCharArray();

		found = false;

		for(i = 0d; i < numericCharacters.Length; i = i + 1d){
			if(numericCharacters[(int)(i)] == c){
				found = true;
			}
		}

		return found;
	}


	public static bool GetJSONPrimitiveName(char [] stringx, double start, StringArrayReference errorMessages, char [] primitive, TokenReference tokenReference){
		char c, p;
		Token token;
		bool done, success;
		double i;
		char [] str;

		token = new Token();
		done = false;
		success = true;

		for(i = start; i < stringx.Length && ((i - start) < primitive.Length) && !done; i = i + 1d){
			c = stringx[(int)(i)];
			p = primitive[(int)(i - start)];
			if(c == p){
				/* OK*/
				if((i + 1d - start) == primitive.Length){
					done = true;
				}
			}else{
				str = "".ToCharArray();
				str = strConcatenateString(str, "Primitive invalid: ".ToCharArray());
				str = strAppendCharacter(str, c);
				str = strAppendString(str, " vs ".ToCharArray());
				str = strAppendCharacter(str, p);

				AddStringRef(errorMessages, CreateStringReference(str));
				done = true;
				success = false;
			}
		}

		if(done){
			if(StringsEqual(primitive, "false".ToCharArray())){
				token = CreateToken(GetTokenType("falseValue".ToCharArray()));
			}
			if(StringsEqual(primitive, "true".ToCharArray())){
				token = CreateToken(GetTokenType("trueValue".ToCharArray()));
			}
			if(StringsEqual(primitive, "null".ToCharArray())){
				token = CreateToken(GetTokenType("nullValue".ToCharArray()));
			}
		}else{
			AddStringRef(errorMessages, CreateStringReference("Primitive invalid".ToCharArray()));
			success = false;
		}

		tokenReference.token = token;

		return success;
	}


	public static bool GetJSONString(char [] json, double start, TokenReference tokenReference, NumberReference stringLengthReference, StringArrayReference errorMessages){
		bool success, done;
		char [] stringx, hex;
		NumberReference characterCount, hexReference;
		double i, l, c;
		StringReference errorMessage;

		characterCount = CreateNumberReference(0d);
		hex = CreateString(4d, '0');
		hexReference = new NumberReference();
		errorMessage = new StringReference();

		success = IsValidJSONStringInJSON(json, start, characterCount, stringLengthReference, errorMessages);

		if(success){
			l = characterCount.numberValue;
			stringx = new char [(int)(l)];

			c = 0d;
			stringx[(int)(c)] = '\"';
			c = c + 1d;

			done = false;
			for(i = start + 1d; !done; i = i + 1d){
				if(json[(int)(i)] == '\\'){
					i = i + 1d;
					if(json[(int)(i)] == '\"' || json[(int)(i)] == '\\' || json[(int)(i)] == '/'){
						stringx[(int)(c)] = json[(int)(i)];
						c = c + 1d;
					}else if(json[(int)(i)] == 'b'){
						stringx[(int)(c)] = (char)(8d);
						c = c + 1d;
					}else if(json[(int)(i)] == 'f'){
						stringx[(int)(c)] = (char)(12d);
						c = c + 1d;
					}else if(json[(int)(i)] == 'n'){
						stringx[(int)(c)] = (char)(10d);
						c = c + 1d;
					}else if(json[(int)(i)] == 'r'){
						stringx[(int)(c)] = (char)(13d);
						c = c + 1d;
					}else if(json[(int)(i)] == 't'){
						stringx[(int)(c)] = (char)(9d);
						c = c + 1d;
					}else if(json[(int)(i)] == 'u'){
						i = i + 1d;
						hex[0] = charToUpperCase(json[(int)(i + 0d)]);
						hex[1] = charToUpperCase(json[(int)(i + 1d)]);
						hex[2] = charToUpperCase(json[(int)(i + 2d)]);
						hex[3] = charToUpperCase(json[(int)(i + 3d)]);
						nCreateNumberFromStringWithCheck(hex, 16d, hexReference, errorMessage);
						stringx[(int)(c)] = (char)(hexReference.numberValue);
						i = i + 3d;
						c = c + 1d;
					}
				}else if(json[(int)(i)] == '\"'){
					stringx[(int)(c)] = json[(int)(i)];
					c = c + 1d;
					done = true;
				}else{
					stringx[(int)(c)] = json[(int)(i)];
					c = c + 1d;
				}
			}

			tokenReference.token = CreateStringToken(stringx);
			success = true;
		}else{
			AddStringRef(errorMessages, CreateStringReference("End of string was not found.".ToCharArray()));
			success = false;
		}

		return success;
	}


	public static bool IsValidJSONString(char [] jsonString, StringArrayReference errorMessages){
		bool valid;
		NumberReference numberReference, stringLength;

		numberReference = new NumberReference();
		stringLength = new NumberReference();

		valid = IsValidJSONStringInJSON(jsonString, 0d, numberReference, stringLength, errorMessages);

		return valid;
	}


	public static bool IsValidJSONStringInJSON(char [] json, double start, NumberReference characterCount, NumberReference stringLengthReference, StringArrayReference errorMessages){
		bool success, done;
		double i, j;
		char c;

		success = true;
		done = false;

		characterCount.numberValue = 1d;

		for(i = start + 1d; i < json.Length && !done && success; i = i + 1d){
			if(!IsJSONIllegalControllCharacter(json[(int)(i)])){
				if(json[(int)(i)] == '\\'){
					i = i + 1d;
					if(i < json.Length){
						if(json[(int)(i)] == '\"' || json[(int)(i)] == '\\' || json[(int)(i)] == '/' || json[(int)(i)] == 'b' || json[(int)(i)] == 'f' || json[(int)(i)] == 'n' || json[(int)(i)] == 'r' || json[(int)(i)] == 't'){
							characterCount.numberValue = characterCount.numberValue + 1d;
						}else if(json[(int)(i)] == 'u'){
							if(i + 4d < json.Length){
								for(j = 0d; j < 4d && success; j = j + 1d){
									c = json[(int)(i + j + 1d)];
									if(nCharacterIsNumberCharacterInBase(c, 16d) || c == 'a' || c == 'b' || c == 'c' || c == 'd' || c == 'e' || c == 'f'){
									}else{
										success = false;
										AddStringRef(errorMessages, CreateStringReference("\\u must be followed by four hexadecimal digits.".ToCharArray()));
									}
								}
								characterCount.numberValue = characterCount.numberValue + 1d;
								i = i + 4d;
							}else{
								success = false;
								AddStringRef(errorMessages, CreateStringReference("\\u must be followed by four characters.".ToCharArray()));
							}
						}else{
							success = false;
							AddStringRef(errorMessages, CreateStringReference("Escaped character invalid.".ToCharArray()));
						}
					}else{
						success = false;
						AddStringRef(errorMessages, CreateStringReference("There must be at least two character after string escape.".ToCharArray()));
					}
				}else if(json[(int)(i)] == '\"'){
					characterCount.numberValue = characterCount.numberValue + 1d;
					done = true;
				}else{
					characterCount.numberValue = characterCount.numberValue + 1d;
				}
			}else{
				success = false;
				AddStringRef(errorMessages, CreateStringReference("Unicode code points 0-31 not allowed in JSON string.".ToCharArray()));
			}
		}

		if(done){
			stringLengthReference.numberValue = i - start;
		}else{
			success = false;
			AddStringRef(errorMessages, CreateStringReference("String must end with \".".ToCharArray()));
		}

		return success;
	}


	public static bool IsJSONIllegalControllCharacter(char c){
		double cnr;
		bool isControll;

		cnr = c;

		if(cnr >= 0d && cnr < 32d){
			isControll = true;
		}else{
			isControll = false;
		}

		return isControll;
	}


	public static Token CreateToken(TokenType tokenType){
		Token token;
		token = new Token();
		token.type = tokenType;
		return token;
	}


	public static Token CreateStringToken(char [] stringx){
		Token token;
		token = new Token();
		token.type = GetTokenType("string".ToCharArray());
		token.value = stringx;
		return token;
	}


	public static Token CreateNumberToken(char [] stringx){
		Token token;
		token = new Token();
		token.type = GetTokenType("number".ToCharArray());
		token.value = stringx;
		return token;
	}


	public static Element [] AddElement(Element [] list, Element a){
		Element [] newlist;
		double i;

		newlist = new Element [(int)(list.Length + 1d)];

		for(i = 0d; i < list.Length; i = i + 1d){
			newlist[(int)(i)] = list[(int)(i)];
		}
		newlist[(int)(list.Length)] = a;

		delete(list);

		return newlist;
	}


	public static void AddElementRef(ElementArrayReference list, Element i){
		list.array = AddElement(list.array, i);
	}


	public static Element [] RemoveElement(Element [] list, double n){
		Element [] newlist;
		double i;

		newlist = new Element [(int)(list.Length - 1d)];

		for(i = 0d; i < list.Length; i = i + 1d){
			if(i < n){
				newlist[(int)(i)] = list[(int)(i)];
			}
			if(i > n){
				newlist[(int)(i - 1d)] = list[(int)(i)];
			}
		}

		delete(list);

		return newlist;
	}


	public static Element GetElementRef(ElementArrayReference list, double i){
		return list.array[(int)(i)];
	}


	public static void RemoveElementRef(ElementArrayReference list, double i){
		list.array = RemoveElement(list.array, i);
	}


	public static double ComputeJSONStringLength(Element element){
		double result;

		result = 0d;

		if(ElementTypeEnumEquals(element.type.name, "object".ToCharArray())){
			result = result + ComputeJSONObjectStringLength(element);
		}else if(ElementTypeEnumEquals(element.type.name, "string".ToCharArray())){
			result = JSONEscapedStringLength(element.stringx) + 2d;
		}else if(ElementTypeEnumEquals(element.type.name, "array".ToCharArray())){
			result = result + ComputeJSONArrayStringLength(element);
		}else if(ElementTypeEnumEquals(element.type.name, "number".ToCharArray())){
			result = result + ComputerJSONNumberStringLength(element);
		}else if(ElementTypeEnumEquals(element.type.name, "nullValue".ToCharArray())){
			result = result + "null".ToCharArray().Length;
		}else if(ElementTypeEnumEquals(element.type.name, "booleanValue".ToCharArray())){
			result = result + ComputeJSONBooleanStringLength(element);
		}

		return result;
	}


	public static double ComputeJSONBooleanStringLength(Element element){
		double result;

		if(element.booleanValue){
			result = "true".ToCharArray().Length;
		}else{
			result = "false".ToCharArray().Length;
		}

		return result;
	}


	public static double ComputerJSONNumberStringLength(Element element){
		double length;

		if(Abs(element.number) >= Pow(10d, 15d) || Abs(element.number) <= Pow(10d, -15d)){
			length = nCreateStringScientificNotationDecimalFromNumber(element.number).Length;
		}else{
			length = nCreateStringDecimalFromNumber(element.number).Length;
		}

		return length;
	}


	public static double ComputeJSONArrayStringLength(Element element){
		Element arrayElement;
		double i;
		double length;

		length = 1d;

		for(i = 0d; i < element.array.Length; i = i + 1d){
			arrayElement = element.array[(int)(i)];

			length = length + ComputeJSONStringLength(arrayElement);

			if(i + 1d != element.array.Length){
				length = length + 1d;
			}
		}

		length = length + 1d;

		return length;
	}


	public static double ComputeJSONObjectStringLength(Element element){
		char [] key;
		double i;
		StringArrayReference keys;
		Element objectElement;
		double length;

		length = 1d;

		keys = GetStringElementMapKeySet(element.objectx);
		for(i = 0d; i < keys.stringArray.Length; i = i + 1d){
			key = keys.stringArray[(int)(i)].stringx;
			objectElement = GetObjectValue(element.objectx, key);

			length = length + 1d;
			length = length + JSONEscapedStringLength(key);
			length = length + 1d;
			length = length + 1d;

			length = length + ComputeJSONStringLength(objectElement);

			if(i + 1d != keys.stringArray.Length){
				length = length + 1d;
			}
		}

		length = length + 1d;

		return length;
	}


	public static Element CreateStringElement(char [] stringx){
		Element element;
		element = new Element();
		element.type = GetElementType("string".ToCharArray());
		element.stringx = stringx;
		return element;
	}


	public static Element CreateBooleanElement(bool booleanValue){
		Element element;
		element = new Element();
		element.type = GetElementType("booleanValue".ToCharArray());
		element.booleanValue = booleanValue;
		return element;
	}


	public static Element CreateNullElement(){
		Element element;
		element = new Element();
		element.type = GetElementType("nullValue".ToCharArray());
		return element;
	}


	public static Element CreateNumberElement(double number){
		Element element;
		element = new Element();
		element.type = GetElementType("number".ToCharArray());
		element.number = number;
		return element;
	}


	public static Element CreateArrayElement(double length){
		Element element;
		element = new Element();
		element.type = GetElementType("array".ToCharArray());
		element.array = new Element [(int)(length)];
		return element;
	}


	public static Element CreateObjectElement(double length){
		Element element;
		element = new Element();
		element.type = GetElementType("object".ToCharArray());
		element.objectx = new StringElementMap();
		element.objectx.stringListRef = CreateStringArrayReferenceLengthValue(length, "".ToCharArray());
		element.objectx.elementListRef = new ElementArrayReference();
		element.objectx.elementListRef.array = new Element [(int)(length)];
		return element;
	}


	public static void DeleteElement(Element element){
		if(ElementTypeEnumEquals(element.type.name, "object".ToCharArray())){
			DeleteObject(element);
		}else if(ElementTypeEnumEquals(element.type.name, "string".ToCharArray())){
			delete(element);
		}else if(ElementTypeEnumEquals(element.type.name, "array".ToCharArray())){
			DeleteArray(element);
		}else if(ElementTypeEnumEquals(element.type.name, "number".ToCharArray())){
			delete(element);
		}else if(ElementTypeEnumEquals(element.type.name, "nullValue".ToCharArray())){
			delete(element);
		}else if(ElementTypeEnumEquals(element.type.name, "booleanValue".ToCharArray())){
			delete(element);
		}else{
		}
	}


	public static void DeleteObject(Element element){
		StringArrayReference keys;
		double i;
		char [] key;
		Element objectElement;

		keys = GetStringElementMapKeySet(element.objectx);
		for(i = 0d; i < keys.stringArray.Length; i = i + 1d){
			key = keys.stringArray[(int)(i)].stringx;
			objectElement = GetObjectValue(element.objectx, key);
			DeleteElement(objectElement);
		}
	}


	public static void DeleteArray(Element element){
		double i;
		Element arrayElement;

		for(i = 0d; i < element.array.Length; i = i + 1d){
			arrayElement = element.array[(int)(i)];
			DeleteElement(arrayElement);
		}
	}


	public static char [] WriteJSON(Element element){
		char [] result;
		double length;
		NumberReference index;

		length = ComputeJSONStringLength(element);
		result = new char [(int)(length)];
		index = CreateNumberReference(0d);

		if(ElementTypeEnumEquals(element.type.name, "object".ToCharArray())){
			WriteObject(element, result, index);
		}else if(ElementTypeEnumEquals(element.type.name, "string".ToCharArray())){
			WriteString(element, result, index);
		}else if(ElementTypeEnumEquals(element.type.name, "array".ToCharArray())){
			WriteArray(element, result, index);
		}else if(ElementTypeEnumEquals(element.type.name, "number".ToCharArray())){
			WriteNumber(element, result, index);
		}else if(ElementTypeEnumEquals(element.type.name, "nullValue".ToCharArray())){
			WriteStringToStingStream(result, index, "null".ToCharArray());
		}else if(ElementTypeEnumEquals(element.type.name, "booleanValue".ToCharArray())){
			WriteBooleanValue(element, result, index);
		}

		return result;
	}


	public static void WriteBooleanValue(Element element, char [] result, NumberReference index){
		if(element.booleanValue){
			WriteStringToStingStream(result, index, "true".ToCharArray());
		}else{
			WriteStringToStingStream(result, index, "false".ToCharArray());
		}
	}


	public static void WriteNumber(Element element, char [] result, NumberReference index){
		char [] numberString;

		if(Abs(element.number) >= Pow(10d, 15d) || Abs(element.number) <= Pow(10d, -15d)){
			numberString = nCreateStringScientificNotationDecimalFromNumber(element.number);
		}else{
			numberString = nCreateStringDecimalFromNumber(element.number);
		}

		WriteStringToStingStream(result, index, numberString);
	}


	public static void WriteArray(Element element, char [] result, NumberReference index){
		char [] s;
		Element arrayElement;
		double i;

		WriteStringToStingStream(result, index, "[".ToCharArray());

		for(i = 0d; i < element.array.Length; i = i + 1d){
			arrayElement = element.array[(int)(i)];

			s = WriteJSON(arrayElement);
			WriteStringToStingStream(result, index, s);

			if(i + 1d != element.array.Length){
				WriteStringToStingStream(result, index, ",".ToCharArray());
			}
		}

		WriteStringToStingStream(result, index, "]".ToCharArray());
	}


	public static void WriteString(Element element, char [] result, NumberReference index){
		WriteStringToStingStream(result, index, "\"".ToCharArray());
		element.stringx = JSONEscapeString(element.stringx);
		WriteStringToStingStream(result, index, element.stringx);
		WriteStringToStingStream(result, index, "\"".ToCharArray());
	}


	public static char [] JSONEscapeString(char [] stringx){
		double i, length;
		NumberReference index, lettersReference;
		char [] ns, escaped;

		length = JSONEscapedStringLength(stringx);

		ns = new char [(int)(length)];
		index = CreateNumberReference(0d);
		lettersReference = CreateNumberReference(0d);

		for(i = 0d; i < stringx.Length; i = i + 1d){
			if(JSONMustBeEscaped(stringx[(int)(i)], lettersReference)){
				escaped = JSONEscapeCharacter(stringx[(int)(i)]);
				WriteStringToStingStream(ns, index, escaped);
			}else{
				WriteCharacterToStingStream(ns, index, stringx[(int)(i)]);
			}
		}

		return ns;
	}


	public static double JSONEscapedStringLength(char [] stringx){
		NumberReference lettersReference;
		double i, length;

		lettersReference = CreateNumberReference(0d);
		length = 0d;

		for(i = 0d; i < stringx.Length; i = i + 1d){
			if(JSONMustBeEscaped(stringx[(int)(i)], lettersReference)){
				length = length + lettersReference.numberValue;
			}else{
				length = length + 1d;
			}
		}
		return length;
	}


	public static char [] JSONEscapeCharacter(char c){
		double code;
		char [] escaped;
		StringReference hexNumber;
		double q, rs, s, b, f, n, r, t;

		code = c;

		q = 34d;
		rs = 92d;
		s = 47d;
		b = 8d;
		f = 12d;
		n = 10d;
		r = 13d;
		t = 9d;

		hexNumber = new StringReference();

		if(code == q){
			escaped = new char [2];
			escaped[0] = '\\';
			escaped[1] = '\"';
		}else if(code == rs){
			escaped = new char [2];
			escaped[0] = '\\';
			escaped[1] = '\\';
		}else if(code == s){
			escaped = new char [2];
			escaped[0] = '\\';
			escaped[1] = '/';
		}else if(code == b){
			escaped = new char [2];
			escaped[0] = '\\';
			escaped[1] = 'b';
		}else if(code == f){
			escaped = new char [2];
			escaped[0] = '\\';
			escaped[1] = 'f';
		}else if(code == n){
			escaped = new char [2];
			escaped[0] = '\\';
			escaped[1] = 'n';
		}else if(code == r){
			escaped = new char [2];
			escaped[0] = '\\';
			escaped[1] = 'r';
		}else if(code == t){
			escaped = new char [2];
			escaped[0] = '\\';
			escaped[1] = 't';
		}else if(code >= 0d && code <= 31d){
			escaped = new char [6];
			escaped[0] = '\\';
			escaped[1] = 'u';
			escaped[2] = '0';
			escaped[3] = '0';

			nCreateStringFromNumberWithCheck(code, 16d, hexNumber);

			if(hexNumber.stringx.Length == 1d){
				escaped[4] = '0';
				escaped[5] = hexNumber.stringx[0];
			}else if(hexNumber.stringx.Length == 2d){
				escaped[4] = hexNumber.stringx[0];
				escaped[5] = hexNumber.stringx[1];
			}
		}else{
			escaped = new char [1];
			escaped[0] = c;
		}

		return escaped;
	}


	public static bool JSONMustBeEscaped(char c, NumberReference letters){
		double code;
		bool mustBeEscaped;
		double q, rs, s, b, f, n, r, t;

		code = c;
		mustBeEscaped = false;

		q = 34d;
		rs = 92d;
		s = 47d;
		b = 8d;
		f = 12d;
		n = 10d;
		r = 13d;
		t = 9d;

		if(code == q || code == rs || code == s || code == b || code == f || code == n || code == r || code == t){
			mustBeEscaped = true;
			letters.numberValue = 2d;
		}else if(code >= 0d && code <= 31d){
			mustBeEscaped = true;
			letters.numberValue = 6d;
		}else if(code >= Pow(2d, 16d)){
			mustBeEscaped = true;
			letters.numberValue = 6d;
		}

		return mustBeEscaped;
	}


	public static void WriteObject(Element element, char [] result, NumberReference index){
		char [] s, key;
		double i;
		StringArrayReference keys;
		Element objectElement;

		WriteStringToStingStream(result, index, "{".ToCharArray());

		keys = GetStringElementMapKeySet(element.objectx);
		for(i = 0d; i < keys.stringArray.Length; i = i + 1d){
			key = keys.stringArray[(int)(i)].stringx;
			key = JSONEscapeString(key);
			objectElement = GetObjectValue(element.objectx, key);

			WriteStringToStingStream(result, index, "\"".ToCharArray());
			WriteStringToStingStream(result, index, key);
			WriteStringToStingStream(result, index, "\"".ToCharArray());
			WriteStringToStingStream(result, index, ":".ToCharArray());

			s = WriteJSON(objectElement);
			WriteStringToStingStream(result, index, s);

			if(i + 1d != keys.stringArray.Length){
				WriteStringToStingStream(result, index, ",".ToCharArray());
			}
		}

		WriteStringToStingStream(result, index, "}".ToCharArray());
	}


	public static void WriteStringToStingStream(char [] stream, NumberReference index, char [] src){
		double i;
		for(i = 0d; i < src.Length; i = i + 1d){
			stream[(int)(index.numberValue + i)] = src[(int)(i)];
		}
		index.numberValue = index.numberValue + src.Length;
	}


	public static void WriteCharacterToStingStream(char [] stream, NumberReference index, char src){
		stream[(int)(index.numberValue)] = src;
		index.numberValue = index.numberValue + 1d;
	}


	public static bool ReadJSON(char [] stringx, ElementReference elementReference, StringArrayReference errorMessages){
		TokenArrayReference tokenArrayReference;
		bool success;

		/* Tokenize.*/
		tokenArrayReference = new TokenArrayReference();
		success = JSONTokenize(stringx, tokenArrayReference, errorMessages);

		if(success){
			/* Parse.*/
			success = GetJSONValue(tokenArrayReference.array, elementReference, errorMessages);
		}

		return success;
	}


	public static bool GetJSONValue(Token [] tokens, ElementReference elementReference, StringArrayReference errorMessages){
		NumberArrayReference counts;
		bool success;
		NumberReference i;

		i = CreateNumberReference(0d);
		counts = CreateNumberArrayReferenceLengthValue(tokens.Length, 0d);

		success = GetJSONValueWithCheckOption(tokens, i, 0d, elementReference, false, counts, errorMessages);

		if(success){
			i.numberValue = 0d;
			GetJSONValueWithCheckOption(tokens, i, 0d, elementReference, true, counts, errorMessages);
		}

		return success;
	}


	public static bool GetJSONValueWithCheckOption(Token [] tokens, NumberReference i, double depth, ElementReference elementReference, bool add, NumberArrayReference counts, StringArrayReference errorMessages){
		Token token;
		char [] str, substr;
		double stringToDecimalResult;
		bool success;

		success = true;
		token = tokens[(int)(i.numberValue)];

		if(TokenTypeEnumEquals(token.type.name, "openCurly".ToCharArray())){
			success = GetJSONObject(tokens, i, depth + 1d, elementReference, add, counts, errorMessages);
		}else if(TokenTypeEnumEquals(token.type.name, "openSquare".ToCharArray())){
			success = GetJSONArray(tokens, i, depth + 1d, elementReference, add, counts, errorMessages);
		}else if(TokenTypeEnumEquals(token.type.name, "trueValue".ToCharArray())){
			if(add){
				elementReference.element = CreateBooleanElement(true);
			}
			i.numberValue = i.numberValue + 1d;
		}else if(TokenTypeEnumEquals(token.type.name, "falseValue".ToCharArray())){
			if(add){
				elementReference.element = CreateBooleanElement(false);
			}
			i.numberValue = i.numberValue + 1d;
		}else if(TokenTypeEnumEquals(token.type.name, "nullValue".ToCharArray())){
			if(add){
				elementReference.element = CreateNullElement();
			}
			i.numberValue = i.numberValue + 1d;
		}else if(TokenTypeEnumEquals(token.type.name, "number".ToCharArray())){
			if(add){
				stringToDecimalResult = nCreateNumberFromDecimalString(token.value);
				elementReference.element = CreateNumberElement(stringToDecimalResult);
			}
			i.numberValue = i.numberValue + 1d;
		}else if(TokenTypeEnumEquals(token.type.name, "string".ToCharArray())){
			if(add){
				substr = strSubstring(token.value, 1d, token.value.Length - 1d);
				elementReference.element = CreateStringElement(substr);
			}
			i.numberValue = i.numberValue + 1d;
		}else{
			str = "".ToCharArray();
			str = strConcatenateString(str, "Invalid token first in value: ".ToCharArray());
			str = strAppendString(str, token.type.name);
			AddStringRef(errorMessages, CreateStringReference(str));
			success = false;
		}

		if(success && depth == 0d){
			if(TokenTypeEnumEquals(tokens[(int)(i.numberValue)].type.name, "end".ToCharArray())){
			}else{
				AddStringRef(errorMessages, CreateStringReference("The outer value cannot have any tokens following it.".ToCharArray()));
				success = false;
			}
		}

		return success;
	}


	public static bool GetJSONObject(Token [] tokens, NumberReference i, double depth, ElementReference elementReference, bool add, NumberArrayReference counts, StringArrayReference errorMessages){
		Element element, value;
		bool done, success;
		Token key, colon, comma, closeCurly;
		char [] keystring, str;
		ElementReference valueReference;
		double countIndex, index;

		countIndex = i.numberValue;
		if(add){
			element = CreateObjectElement(counts.numberArray[(int)(countIndex)]);
		}else{
			element = new Element();
		}
		valueReference = new ElementReference();
		success = true;
		i.numberValue = i.numberValue + 1d;
		index = 0d;

		if(!TokenTypeEnumEquals(tokens[(int)(i.numberValue)].type.name, "closeCurly".ToCharArray())){
			done = false;

			for(; !done && success; ){
				key = tokens[(int)(i.numberValue)];

				if(TokenTypeEnumEquals(key.type.name, "string".ToCharArray())){
					i.numberValue = i.numberValue + 1d;
					colon = tokens[(int)(i.numberValue)];
					if(TokenTypeEnumEquals(colon.type.name, "colon".ToCharArray())){
						i.numberValue = i.numberValue + 1d;
						success = GetJSONValueWithCheckOption(tokens, i, depth, valueReference, add, counts, errorMessages);

						if(success){
							value = valueReference.element;

							keystring = strSubstring(key.value, 1d, key.value.Length - 1d);
							if(add){
								SetStringElementMap(element.objectx, index, keystring, value);
							}

							index = index + 1d;

							comma = tokens[(int)(i.numberValue)];
							if(TokenTypeEnumEquals(comma.type.name, "comma".ToCharArray())){
								/* OK*/
								i.numberValue = i.numberValue + 1d;
							}else{
								done = true;
							}
						}
					}else{
						str = "".ToCharArray();
						str = strConcatenateString(str, "Expected colon after key in object: ".ToCharArray());
						str = strAppendString(str, colon.type.name);
						AddStringRef(errorMessages, CreateStringReference(str));

						success = false;
						done = true;
					}
				}else{
					AddStringRef(errorMessages, CreateStringReference("Expected string as key in object.".ToCharArray()));

					done = true;
					success = false;
				}
			}
		}

		if(success){
			closeCurly = tokens[(int)(i.numberValue)];

			if(TokenTypeEnumEquals(closeCurly.type.name, "closeCurly".ToCharArray())){
				/* OK*/
				elementReference.element = element;
				i.numberValue = i.numberValue + 1d;
			}else{
				AddStringRef(errorMessages, CreateStringReference("Expected close curly brackets at end of object value.".ToCharArray()));
				success = false;
			}
		}

		counts.numberArray[(int)(countIndex)] = index;

		return success;
	}


	public static bool GetJSONArray(Token [] tokens, NumberReference i, double depth, ElementReference elementReference, bool add, NumberArrayReference counts, StringArrayReference errorMessages){
		Element element, value;
		Token nextToken, comma;
		bool done, success;
		ElementReference valueReference;
		double countIndex, index;

		index = 0d;
		countIndex = i.numberValue;
		i.numberValue = i.numberValue + 1d;

		if(add){
			element = CreateArrayElement(counts.numberArray[(int)(countIndex)]);
		}else{
			element = new Element();
		}
		valueReference = new ElementReference();
		success = true;

		nextToken = tokens[(int)(i.numberValue)];

		if(!TokenTypeEnumEquals(nextToken.type.name, "closeSquare".ToCharArray())){
			done = false;
			for(; !done && success; ){
				success = GetJSONValueWithCheckOption(tokens, i, depth, valueReference, add, counts, errorMessages);

				if(success){
					value = valueReference.element;

					if(add){
						element.array[(int)(index)] = value;
					}

					index = index + 1d;

					comma = tokens[(int)(i.numberValue)];

					if(TokenTypeEnumEquals(comma.type.name, "comma".ToCharArray())){
						/* OK*/
						i.numberValue = i.numberValue + 1d;
					}else{
						done = true;
					}
				}
			}
		}

		nextToken = tokens[(int)(i.numberValue)];
		if(TokenTypeEnumEquals(nextToken.type.name, "closeSquare".ToCharArray())){
			/* OK*/
			i.numberValue = i.numberValue + 1d;
			elementReference.element = element;
		}else{
			AddStringRef(errorMessages, CreateStringReference("Expected close square bracket at end of array.".ToCharArray()));
			success = false;
		}

		elementReference.element = element;
		counts.numberArray[(int)(countIndex)] = index;

		return success;
	}


	public static StringArrayReference GetStringElementMapKeySet(StringElementMap stringElementMap){
		return stringElementMap.stringListRef;
	}


	public static Element GetObjectValue(StringElementMap stringElementMap, char [] key){
		Element result;
		double i;

		result = new Element();

		for(i = 0d; i < GetStringElementMapNumberOfKeys(stringElementMap); i = i + 1d){
			if(StringsEqual(stringElementMap.stringListRef.stringArray[(int)(i)].stringx, key)){
				result = stringElementMap.elementListRef.array[(int)(i)];
			}
		}

		return result;
	}


	public static Element GetObjectValueWithCheck(StringElementMap stringElementMap, char [] key, BooleanReference foundReference){
		Element result;
		double i;

		result = new Element();

		foundReference.booleanValue = false;
		for(i = 0d; i < GetStringElementMapNumberOfKeys(stringElementMap); i = i + 1d){
			if(StringsEqual(stringElementMap.stringListRef.stringArray[(int)(i)].stringx, key)){
				result = stringElementMap.elementListRef.array[(int)(i)];
				foundReference.booleanValue = true;
			}
		}

		return result;
	}


	public static void PutStringElementMap(StringElementMap stringElementMap, char [] keystring, Element value){
		AddStringRef(stringElementMap.stringListRef, CreateStringReference(keystring));
		AddElementRef(stringElementMap.elementListRef, value);
	}


	public static void SetStringElementMap(StringElementMap stringElementMap, double index, char [] keystring, Element value){
		stringElementMap.stringListRef.stringArray[(int)(index)].stringx = keystring;
		stringElementMap.elementListRef.array[(int)(index)] = value;
	}


	public static double GetStringElementMapNumberOfKeys(StringElementMap stringElementMap){
		return stringElementMap.stringListRef.stringArray.Length;
	}


	public static TokenType GetTokenType(char [] elementTypeName){
		TokenType et;

		et = new TokenType();
		et.name = elementTypeName;

		return et;
	}


	public static TokenType GetAndCheckTokenType(char [] elementTypeName, BooleanReference found){
		TokenType [] elementTypes;
		TokenType tokenType;
		double i, count;

		count = 12d;

		elementTypes = new TokenType [(int)(count)];

		for(i = 0d; i < count; i = i + 1d){
			elementTypes[(int)(i)] = new TokenType();
		}

		elementTypes[0].name = "openCurly".ToCharArray();
		elementTypes[1].name = "closeCurly".ToCharArray();
		elementTypes[2].name = "openSquare".ToCharArray();
		elementTypes[3].name = "closeSquare".ToCharArray();
		elementTypes[4].name = "comma".ToCharArray();
		elementTypes[5].name = "colon".ToCharArray();
		elementTypes[6].name = "nullValue".ToCharArray();
		elementTypes[7].name = "trueValue".ToCharArray();
		elementTypes[8].name = "falseValue".ToCharArray();
		elementTypes[9].name = "string".ToCharArray();
		elementTypes[10].name = "number".ToCharArray();
		elementTypes[11].name = "end".ToCharArray();

		found.booleanValue = false;
		tokenType = new TokenType();
		for(i = 0d; i < count && !found.booleanValue; i = i + 1d){
			tokenType = elementTypes[(int)(i)];
			if(StringsEqual(tokenType.name, elementTypeName)){
				found.booleanValue = true;
			}
		}

		return tokenType;
	}


	public static bool TokenTypeEnumStructureEquals(TokenType a, TokenType b){
		return StringsEqual(a.name, b.name);
	}


	public static bool TokenTypeEnumEquals(char [] a, char [] b){
		bool equals;
		TokenType eta, etb;
		BooleanReference founda, foundb;

		founda = new BooleanReference();
		foundb = new BooleanReference();

		eta = GetAndCheckTokenType(a, founda);
		etb = GetAndCheckTokenType(b, foundb);

		if(founda.booleanValue && foundb.booleanValue){
			equals = TokenTypeEnumStructureEquals(eta, etb);
		}else{
			equals = false;
		}

		return equals;
	}


	public static bool JSONCompare(char [] a, char [] b, double epsilon, BooleanReference equal, StringArrayReference errorMessage){
		bool success;
		ElementReference eaRef, ebRef;
		Element ea, eb;

		eaRef = new ElementReference();
		ebRef = new ElementReference();

		success = ReadJSON(a, eaRef, errorMessage);

		if(success){
			ea = eaRef.element;

			success = ReadJSON(b, ebRef, errorMessage);

			if(success){
				eb = ebRef.element;

				equal.booleanValue = JSONCompareElements(ea, eb, epsilon);

				DeleteElement(eb);
			}

			DeleteElement(ea);
		}

		return success;
	}


	public static bool JSONCompareElements(Element ea, Element eb, double epsilon){
		bool equal;
		char [] typeName;

		equal = StringsEqual(ea.type.name, eb.type.name);
        
		if(equal){
			typeName = ea.type.name;
			if(ElementTypeEnumEquals(typeName, "object".ToCharArray())){
				equal = JSONCompareObjects(ea.objectx, eb.objectx, epsilon);
			}else if(ElementTypeEnumEquals(typeName, "string".ToCharArray())){
				equal = StringsEqual(ea.stringx, eb.stringx);
			}else if(ElementTypeEnumEquals(typeName, "array".ToCharArray())){
				equal = JSONCompareArrays(ea.array, eb.array, epsilon);
			}else if(ElementTypeEnumEquals(typeName, "number".ToCharArray())){
				equal = EpsilonCompare(ea.number, eb.number, epsilon);
			}else if(ElementTypeEnumEquals(typeName, "nullValue".ToCharArray())){
				equal = true;
			}else if(ElementTypeEnumEquals(typeName, "booleanValue".ToCharArray())){
				equal = ea.booleanValue == eb.booleanValue;
			}
		}
        
		return equal;
	}


	public static bool JSONCompareArrays(Element [] ea, Element [] eb, double epsilon){
		bool equals;
		double i, length;

		equals = ea.Length == eb.Length;

		if(equals){
			length = ea.Length;
			for(i = 0d; i < length && equals; i = i + 1d){
				equals = JSONCompareElements(ea[(int)(i)], eb[(int)(i)], epsilon);
			}
		}

		return equals;
	}


	public static bool JSONCompareObjects(StringElementMap ea, StringElementMap eb, double epsilon){
		bool equals;
		double akeys, bkeys, i;
		StringArrayReference keys;
		char [] key;
		BooleanReference aFoundReference, bFoundReference;
		Element eaValue, ebValue;

		aFoundReference = new BooleanReference();
		bFoundReference = new BooleanReference();

		akeys = GetStringElementMapNumberOfKeys(ea);
		bkeys = GetStringElementMapNumberOfKeys(eb);

		equals = akeys == bkeys;

		if(equals){
			keys = GetStringElementMapKeySet(ea);

			for(i = 0d; i < keys.stringArray.Length && equals; i = i + 1d){
				key = keys.stringArray[(int)(i)].stringx;

				eaValue = GetObjectValueWithCheck(ea, key, aFoundReference);
				ebValue = GetObjectValueWithCheck(eb, key, bFoundReference);

				if(aFoundReference.booleanValue && bFoundReference.booleanValue){
					equals = JSONCompareElements(eaValue, ebValue, epsilon);
				}else{
					equals = false;
				}
			}
		}

		return equals;
	}


	public static ElementType GetElementType(char [] elementTypeName){
		ElementType et;

		et = new ElementType();
		et.name = elementTypeName;

		return et;
	}


	public static ElementType GetAndCheckElementType(char [] elementTypeName, BooleanReference found){
		ElementType [] elementTypes;
		ElementType elementType;
		double i, antall;

		antall = 6d;

		elementTypes = new ElementType [(int)(antall)];

		for(i = 0d; i < antall; i = i + 1d){
			elementTypes[(int)(i)] = new ElementType();
		}

		elementTypes[0].name = "object".ToCharArray();
		elementTypes[1].name = "array".ToCharArray();
		elementTypes[2].name = "string".ToCharArray();
		elementTypes[3].name = "number".ToCharArray();
		elementTypes[4].name = "booleanValue".ToCharArray();
		elementTypes[5].name = "nullValue".ToCharArray();

		found.booleanValue = false;
		elementType = new ElementType();
		for(i = 0d; i < antall && !found.booleanValue; i = i + 1d){
			elementType = elementTypes[(int)(i)];
			if(StringsEqual(elementType.name, elementTypeName)){
				found.booleanValue = true;
			}
		}

		return elementType;
	}


	public static bool ElementTypeStructureEquals(ElementType a, ElementType b){
		return StringsEqual(a.name, b.name);
	}


	public static bool ElementTypeEnumEquals(char [] a, char [] b){
		bool equals;
		ElementType eta, etb;
		BooleanReference founda, foundb;

		founda = new BooleanReference();
		foundb = new BooleanReference();

		eta = GetAndCheckElementType(a, founda);
		etb = GetAndCheckElementType(b, foundb);

		if(founda.booleanValue && foundb.booleanValue){
			equals = ElementTypeStructureEquals(eta, etb);
		}else{
			equals = false;
		}

		return equals;
	}


	public static BooleanReference CreateBooleanReference(bool value){
		BooleanReference refx;
		refx = new BooleanReference();
		refx.booleanValue = value;

		return refx;
	}


	public static BooleanArrayReference CreateBooleanArrayReference(bool [] value){
		BooleanArrayReference refx;
		refx = new BooleanArrayReference();
		refx.booleanArray = value;

		return refx;
	}


	public static BooleanArrayReference CreateBooleanArrayReferenceLengthValue(double length, bool value){
		BooleanArrayReference refx;
		double i;
		refx = new BooleanArrayReference();
		refx.booleanArray = new bool [(int)(length)];

		for(i = 0d; i < length; i = i + 1d){
			refx.booleanArray[(int)(i)] = value;
		}

		return refx;
	}


	public static void FreeBooleanArrayReference(BooleanArrayReference booleanArrayReference){
		delete(booleanArrayReference.booleanArray);
		delete(booleanArrayReference);
	}


	public static CharacterReference CreateCharacterReference(char value){
		CharacterReference refx;
		refx = new CharacterReference();
		refx.characterValue = value;

		return refx;
	}


	public static NumberReference CreateNumberReference(double value){
		NumberReference refx;
		refx = new NumberReference();
		refx.numberValue = value;

		return refx;
	}


	public static NumberArrayReference CreateNumberArrayReference(double [] value){
		NumberArrayReference refx;
		refx = new NumberArrayReference();
		refx.numberArray = value;

		return refx;
	}


	public static NumberArrayReference CreateNumberArrayReferenceLengthValue(double length, double value){
		NumberArrayReference refx;
		double i;
		refx = new NumberArrayReference();
		refx.numberArray = new double [(int)(length)];

		for(i = 0d; i < length; i = i + 1d){
			refx.numberArray[(int)(i)] = value;
		}

		return refx;
	}


	public static void FreeNumberArrayReference(NumberArrayReference numberArrayReference){
		delete(numberArrayReference.numberArray);
		delete(numberArrayReference);
	}


	public static StringReference CreateStringReference(char [] value){
		StringReference refx;
		refx = new StringReference();
		refx.stringx = value;

		return refx;
	}


	public static StringReference CreateStringReferenceLengthValue(double length, char value){
		StringReference refx;
		double i;
		refx = new StringReference();
		refx.stringx = new char [(int)(length)];

		for(i = 0d; i < length; i = i + 1d){
			refx.stringx[(int)(i)] = value;
		}

		return refx;
	}


	public static void FreeStringReference(StringReference stringReference){
		delete(stringReference.stringx);
		delete(stringReference);
	}


	public static StringArrayReference CreateStringArrayReference(StringReference [] strings){
		StringArrayReference refx;
		refx = new StringArrayReference();
		refx.stringArray = strings;

		return refx;
	}


	public static StringArrayReference CreateStringArrayReferenceLengthValue(double length, char [] value){
		StringArrayReference refx;
		double i;
		refx = new StringArrayReference();
		refx.stringArray = new StringReference [(int)(length)];

		for(i = 0d; i < length; i = i + 1d){
			refx.stringArray[(int)(i)] = CreateStringReference(value);
		}

		return refx;
	}


	public static void FreeStringArrayReference(StringArrayReference stringArrayReference){
		double i;
		for(i = 0d; i < stringArrayReference.stringArray.Length; i = i + 1d){
			delete(stringArrayReference.stringArray[(int)(i)]);
		}
		delete(stringArrayReference.stringArray);
		delete(stringArrayReference);
	}


	public static bool strSubstringWithCheck(char [] stringx, double from, double to, StringReference stringReference){
		bool success;
		if(from < stringx.Length && to < stringx.Length && from <= to && from >= 0d && to >= 0d){
			stringReference.stringx = strSubstring(stringx, from, to);
			success = true;
		}else{
			success = false;
		}

		return success;
	}


	public static char [] strSubstring(char [] stringx, double from, double to){
		char [] n;
		double i;
		n = new char [(int)(Max(to - from, 0d))];

		for(i = from; i < to; i = i + 1d){
			n[(int)(i - from)] = stringx[(int)(i)];
		}

		return n;
	}


	public static char [] strAppendString(char [] stringx, char [] s){
		char [] newString;
		newString = strConcatenateString(stringx, s);

		delete(stringx);

		return newString;
	}


	public static char [] strConcatenateString(char [] stringx, char [] s){
		char [] newString;
		double i;
		newString = new char [(int)(stringx.Length + s.Length)];

		for(i = 0d; i < stringx.Length; i = i + 1d){
			newString[(int)(i)] = stringx[(int)(i)];
		}

		for(i = 0d; i < s.Length; i = i + 1d){
			newString[(int)(stringx.Length + i)] = s[(int)(i)];
		}

		return newString;
	}


	public static char [] strAppendCharacter(char [] stringx, char c){
		char [] newString;
		newString = strConcatenateCharacter(stringx, c);

		delete(stringx);

		return newString;
	}


	public static char [] strConcatenateCharacter(char [] stringx, char c){
		char [] newString;
		double i;
		newString = new char [(int)(stringx.Length + 1d)];

		for(i = 0d; i < stringx.Length; i = i + 1d){
			newString[(int)(i)] = stringx[(int)(i)];
		}

		newString[(int)(stringx.Length)] = c;

		return newString;
	}


	public static StringReference [] strSplitByCharacter(char [] toSplit, char splitBy){
		StringReference [] split;
		char [] stringToSplitBy;
		stringToSplitBy = new char [1];
		stringToSplitBy[0] = splitBy;

		split = strSplitByString(toSplit, stringToSplitBy);

		delete(stringToSplitBy);

		return split;
	}


	public static bool strIndexOfCharacter(char [] stringx, char character, NumberReference indexReference){
		double i;
		bool found;
		found = false;
		for(i = 0d; i < stringx.Length && !found; i = i + 1d){
			if(stringx[(int)(i)] == character){
				found = true;
				indexReference.numberValue = i;
			}
		}

		return found;
	}


	public static bool strSubstringEqualsWithCheck(char [] stringx, double from, char [] substring, BooleanReference equalsReference){
		bool success;
		if(from < stringx.Length){
			success = true;
			equalsReference.booleanValue = strSubstringEquals(stringx, from, substring);
		}else{
			success = false;
		}

		return success;
	}


	public static bool strSubstringEquals(char [] stringx, double from, char [] substring){
		double i;
		bool equal;
		equal = true;
		for(i = 0d; i < substring.Length && equal; i = i + 1d){
			if(stringx[(int)(from + i)] != substring[(int)(i)]){
				equal = false;
			}
		}

		return equal;
	}


	public static bool strIndexOfString(char [] stringx, char [] substring, NumberReference indexReference){
		double i;
		bool found;
		found = false;
		for(i = 0d; i < stringx.Length - substring.Length + 1d && !found; i = i + 1d){
			if(strSubstringEquals(stringx, i, substring)){
				found = true;
				indexReference.numberValue = i;
			}
		}

		return found;
	}


	public static bool strContainsCharacter(char [] stringx, char character){
		return strIndexOfCharacter(stringx, character, new NumberReference());
	}


	public static bool strContainsString(char [] stringx, char [] substring){
		return strIndexOfString(stringx, substring, new NumberReference());
	}


	public static void strToUpperCase(char [] stringx){
		double i;
		for(i = 0d; i < stringx.Length; i = i + 1d){
			stringx[(int)(i)] = charToUpperCase(stringx[(int)(i)]);
		}
	}


	public static void strToLowerCase(char [] stringx){
		double i;
		for(i = 0d; i < stringx.Length; i = i + 1d){
			stringx[(int)(i)] = charToLowerCase(stringx[(int)(i)]);
		}
	}


	public static bool strEqualsIgnoreCase(char [] a, char [] b){
		bool equal;
		double i;
		if(a.Length == b.Length){
			equal = true;
			for(i = 0d; i < a.Length && equal; i = i + 1d){
				if(charToLowerCase(a[(int)(i)]) != charToLowerCase(b[(int)(i)])){
					equal = false;
				}
			}
		}else{
			equal = false;
		}

		return equal;
	}


	public static char [] strReplacesString(char [] stringx, char [] toReplace, char [] replaceWith){
		char [] result;
		double i;
		BooleanReference equalsReference;
		bool success;
		equalsReference = new BooleanReference();
		result = new char [0];

		for(i = 0d; i < stringx.Length; ){
			success = strSubstringEqualsWithCheck(stringx, i, toReplace, equalsReference);
			if(success){
				success = equalsReference.booleanValue;
			}
			if(success && toReplace.Length > 0d){
				result = strConcatenateString(result, replaceWith);
				i = i + toReplace.Length;
			}else{
				result = strConcatenateCharacter(result, stringx[(int)(i)]);
				i = i + 1d;
			}
		}

		return result;
	}


	public static char [] strReplaceCharacter(char [] stringx, char toReplace, char replaceWith){
		char [] result;
		double i;
		result = new char [0];

		for(i = 0d; i < stringx.Length; i = i + 1d){
			if(stringx[(int)(i)] == toReplace){
				result = strConcatenateCharacter(result, replaceWith);
			}else{
				result = strConcatenateCharacter(result, stringx[(int)(i)]);
			}
		}

		return result;
	}


	public static char [] strTrim(char [] stringx){
		char [] result;
		double i, lastWhitespaceLocationStart, lastWhitespaceLocationEnd;
		bool firstNonWhitespaceFound;
		lastWhitespaceLocationStart = -1d;
		firstNonWhitespaceFound = false;
		for(i = 0d; i < stringx.Length && !firstNonWhitespaceFound; i = i + 1d){
			if(charIsWhiteSpace(stringx[(int)(i)])){
				lastWhitespaceLocationStart = i;
			}else{
				firstNonWhitespaceFound = true;
			}
		}

		/* Find whitepaces at the end.*/
		lastWhitespaceLocationEnd = stringx.Length;
		firstNonWhitespaceFound = false;
		for(i = stringx.Length - 1d; i >= 0d && !firstNonWhitespaceFound; i = i - 1d){
			if(charIsWhiteSpace(stringx[(int)(i)])){
				lastWhitespaceLocationEnd = i;
			}else{
				firstNonWhitespaceFound = true;
			}
		}

		if(lastWhitespaceLocationStart < lastWhitespaceLocationEnd){
			result = strSubstring(stringx, lastWhitespaceLocationStart + 1d, lastWhitespaceLocationEnd);
		}else{
			result = new char [0];
		}

		return result;
	}


	public static bool strStartsWith(char [] stringx, char [] start){
		bool startsWithString;
		startsWithString = false;
		if(stringx.Length >= start.Length){
			startsWithString = strSubstringEquals(stringx, 0d, start);
		}

		return startsWithString;
	}


	public static bool strEndsWith(char [] stringx, char [] end){
		bool endsWithString;
		endsWithString = false;
		if(stringx.Length >= end.Length){
			endsWithString = strSubstringEquals(stringx, stringx.Length - end.Length, end);
		}

		return endsWithString;
	}


	public static StringReference [] strSplitByString(char [] toSplit, char [] splitBy){
		StringReference [] split;
		char [] next;
		double i;
		char c;
		StringReference n;
		split = new StringReference [0];

		next = new char [0];
		for(i = 0d; i < toSplit.Length; ){
			c = toSplit[(int)(i)];
			if(strSubstringEquals(toSplit, i, splitBy)){
				if(split.Length != 0d || i != 0d){
					n = new StringReference();
					n.stringx = next;
					split = AddString(split, n);
					next = new char [0];
					i = i + splitBy.Length;
				}
			}else{
				next = strAppendCharacter(next, c);
				i = i + 1d;
			}
		}

		if(next.Length > 0d){
			n = new StringReference();
			n.stringx = next;
			split = AddString(split, n);
		}

		return split;
	}


	public static char [] nCreateStringScientificNotationDecimalFromNumber(double decimalx){
		StringReference mantissaReference, exponentReference;
		double multiplier, inc;
		double exponent;
		bool done, isPositive;
		char [] result;
		mantissaReference = new StringReference();
		exponentReference = new StringReference();
		result = new char [0];
		done = false;
		exponent = 0d;

		if(decimalx < 0d){
			isPositive = false;
			decimalx = -decimalx;
		}else{
			isPositive = true;
		}

		if(decimalx == 0d){
			done = true;
		}

		if(!done){
			multiplier = 0d;
			inc = 0d;
			if(decimalx < 1d){
				multiplier = 10d;
				inc = -1d;
			}else if(decimalx >= 10d){
				multiplier = 0.1;
				inc = 1d;
			}else{
				done = true;
			}
			if(!done){
				for(; decimalx >= 10d || decimalx < 1d; ){
					decimalx = decimalx*multiplier;
					exponent = exponent + inc;
				}
			}
		}

		nCreateStringFromNumberWithCheck(decimalx, 10d, mantissaReference);

		nCreateStringFromNumberWithCheck(exponent, 10d, exponentReference);

		if(!isPositive){
			result = strAppendString(result, "-".ToCharArray());
		}

		result = strAppendString(result, mantissaReference.stringx);
		result = strAppendString(result, "e".ToCharArray());
		result = strAppendString(result, exponentReference.stringx);

		return result;
	}


	public static char [] nCreateStringDecimalFromNumber(double decimalx){
		StringReference stringReference;
		stringReference = new StringReference();

		/* This will succeed because base = 10.*/
		nCreateStringFromNumberWithCheck(decimalx, 10d, stringReference);

		return stringReference.stringx;
	}


	public static bool nCreateStringFromNumberWithCheck(double decimalx, double basex, StringReference stringReference){
		char [] stringx;
		double maximumDigits;
		double digitPosition;
		bool hasPrintedPoint, isPositive;
		double i, d;
		bool success;
		CharacterReference characterReference;
		char c;
		isPositive = true;

		if(decimalx < 0d){
			isPositive = false;
			decimalx = -decimalx;
		}

		if(decimalx == 0d){
			stringReference.stringx = "0".ToCharArray();
			success = true;
		}else{
			characterReference = new CharacterReference();
			if(IsInteger(basex)){
				success = true;
				stringx = new char [0];
				maximumDigits = nGetMaximumDigitsForBase(basex);
				digitPosition = nGetFirstDigitPosition(decimalx, basex);
				decimalx = (double)Round(decimalx*Pow(basex, maximumDigits - digitPosition - 1d));
				hasPrintedPoint = false;
				if(!isPositive){
					stringx = strAppendCharacter(stringx, '-');
				}
				if(digitPosition < 0d){
					stringx = strAppendCharacter(stringx, '0');
					stringx = strAppendCharacter(stringx, '.');
					hasPrintedPoint = true;
					for(i = 0d; i < -digitPosition - 1d; i = i + 1d){
						stringx = strAppendCharacter(stringx, '0');
					}
				}
				for(i = 0d; i < maximumDigits && success; i = i + 1d){
					d = Floor(decimalx/Pow(basex, maximumDigits - i - 1d));
					if(d >= basex){
						d = basex - 1d;
					}
					if(!hasPrintedPoint && digitPosition - i + 1d == 0d){
						if(decimalx != 0d){
							stringx = strAppendCharacter(stringx, '.');
						}
						hasPrintedPoint = true;
					}
					if(decimalx == 0d && hasPrintedPoint){
					}else{
						success = nGetSingleDigitCharacterFromNumberWithCheck(d, basex, characterReference);
						if(success){
							c = characterReference.characterValue;
							stringx = strAppendCharacter(stringx, c);
						}
					}
					if(success){
						decimalx = decimalx - d*Pow(basex, maximumDigits - i - 1d);
					}
				}
				if(success){
					for(i = 0d; i < digitPosition - maximumDigits + 1d; i = i + 1d){
						stringx = strAppendCharacter(stringx, '0');
					}
					stringReference.stringx = stringx;
				}
			}else{
				success = false;
			}
		}

		/* Done*/
		return success;
	}


	public static double nGetMaximumDigitsForBase(double basex){
		double t;
		t = Pow(10d, 15d);
		return Floor(Log10(t)/Log10(basex));
	}


	public static double nGetFirstDigitPosition(double decimalx, double basex){
		double power;
		double t;
		power = Ceiling(Log10(decimalx)/Log10(basex));

		t = decimalx*Pow(basex, -power);
		if(t < basex && t >= 1d){
		}else if(t >= basex){
			power = power + 1d;
		}else if(t < 1d){
			power = power - 1d;
		}

		return power;
	}


	public static bool nGetSingleDigitCharacterFromNumberWithCheck(double c, double basex, CharacterReference characterReference){
		char [] numberTable;
		bool success;
		numberTable = nGetDigitCharacterTable();

		if(c < basex || c < numberTable.Length){
			success = true;
			characterReference.characterValue = numberTable[(int)(c)];
		}else{
			success = false;
		}

		return success;
	}


	public static char [] nGetDigitCharacterTable(){
		char [] numberTable;
		numberTable = "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ".ToCharArray();

		return numberTable;
	}


	public static bool nCreateNumberFromDecimalStringWithCheck(char [] stringx, NumberReference decimalReference, StringReference errorMessage){
		return nCreateNumberFromStringWithCheck(stringx, 10d, decimalReference, errorMessage);
	}


	public static double nCreateNumberFromDecimalString(char [] stringx){
		NumberReference doubleReference;
		StringReference stringReference;
		double number;
		doubleReference = CreateNumberReference(0d);
		stringReference = CreateStringReference("".ToCharArray());
		nCreateNumberFromStringWithCheck(stringx, 10d, doubleReference, stringReference);
		number = doubleReference.numberValue;

		delete(doubleReference);
		delete(stringReference);

		return number;
	}


	public static bool nCreateNumberFromStringWithCheck(char [] stringx, double basex, NumberReference numberReference, StringReference errorMessage){
		bool success;
		BooleanReference numberIsPositive, exponentIsPositive;
		NumberArrayReference beforePoint, afterPoint, exponent;
		numberIsPositive = CreateBooleanReference(true);
		exponentIsPositive = CreateBooleanReference(true);
		beforePoint = new NumberArrayReference();
		afterPoint = new NumberArrayReference();
		exponent = new NumberArrayReference();

		if(basex >= 2d && basex <= 36d){
			success = nExtractPartsFromNumberString(stringx, basex, numberIsPositive, beforePoint, afterPoint, exponentIsPositive, exponent, errorMessage);
			if(success){
				numberReference.numberValue = nCreateNumberFromParts(basex, numberIsPositive.booleanValue, beforePoint.numberArray, afterPoint.numberArray, exponentIsPositive.booleanValue, exponent.numberArray);
			}
		}else{
			success = false;
			errorMessage.stringx = "Base must be from 2 to 36.".ToCharArray();
		}

		return success;
	}


	public static double nCreateNumberFromParts(double basex, bool numberIsPositive, double [] beforePoint, double [] afterPoint, bool exponentIsPositive, double [] exponent){
		double n, i, p, e;
		n = 0d;

		for(i = 0d; i < beforePoint.Length; i = i + 1d){
			p = beforePoint[(int)(beforePoint.Length - i - 1d)];
			n = n + p*Pow(basex, i);
		}

		for(i = 0d; i < afterPoint.Length; i = i + 1d){
			p = afterPoint[(int)(i)];
			n = n + p*Pow(basex, -(i + 1d));
		}

		if(exponent.Length > 0d){
			e = 0d;
			for(i = 0d; i < exponent.Length; i = i + 1d){
				p = exponent[(int)(exponent.Length - i - 1d)];
				e = e + p*Pow(basex, i);
			}
			if(!exponentIsPositive){
				e = -e;
			}
			n = n*Pow(basex, e);
		}

		if(!numberIsPositive){
			n = -n;
		}

		return n;
	}


	public static bool nExtractPartsFromNumberString(char [] n, double basex, BooleanReference numberIsPositive, NumberArrayReference beforePoint, NumberArrayReference afterPoint, BooleanReference exponentIsPositive, NumberArrayReference exponent, StringReference errorMessages){
		double i;
		bool success;
		i = 0d;

		if(i < n.Length){
			if(n[(int)(i)] == '-'){
				numberIsPositive.booleanValue = false;
				i = i + 1d;
			}else if(n[(int)(i)] == '+'){
				numberIsPositive.booleanValue = true;
				i = i + 1d;
			}
			success = nExtractPartsFromNumberStringFromSign(n, basex, i, beforePoint, afterPoint, exponentIsPositive, exponent, errorMessages);
		}else{
			success = false;
			errorMessages.stringx = "Number cannot have length zero.".ToCharArray();
		}

		return success;
	}


	public static bool nExtractPartsFromNumberStringFromSign(char [] n, double basex, double i, NumberArrayReference beforePoint, NumberArrayReference afterPoint, BooleanReference exponentIsPositive, NumberArrayReference exponent, StringReference errorMessages){
		bool success, done;
		double count, j;
		done = false;
		count = 0d;
		for(; i + count < n.Length && !done; ){
			if(nCharacterIsNumberCharacterInBase(n[(int)(i + count)], basex)){
				count = count + 1d;
			}else{
				done = true;
			}
		}

		if(count >= 1d){
			beforePoint.numberArray = new double [(int)(count)];
			for(j = 0d; j < count; j = j + 1d){
				beforePoint.numberArray[(int)(j)] = nGetNumberFromNumberCharacterForBase(n[(int)(i + j)], basex);
			}
			i = i + count;
			if(i < n.Length){
				success = nExtractPartsFromNumberStringFromPointOrExponent(n, basex, i, afterPoint, exponentIsPositive, exponent, errorMessages);
			}else{
				afterPoint.numberArray = new double [0];
				exponent.numberArray = new double [0];
				success = true;
			}
		}else{
			success = false;
			errorMessages.stringx = "Number must have at least one number after the optional sign.".ToCharArray();
		}

		return success;
	}


	public static bool nExtractPartsFromNumberStringFromPointOrExponent(char [] n, double basex, double i, NumberArrayReference afterPoint, BooleanReference exponentIsPositive, NumberArrayReference exponent, StringReference errorMessages){
		bool success, done;
		double count, j;
		if(n[(int)(i)] == '.'){
			i = i + 1d;
			if(i < n.Length){
				done = false;
				count = 0d;
				for(; i + count < n.Length && !done; ){
					if(nCharacterIsNumberCharacterInBase(n[(int)(i + count)], basex)){
						count = count + 1d;
					}else{
						done = true;
					}
				}
				if(count >= 1d){
					afterPoint.numberArray = new double [(int)(count)];
					for(j = 0d; j < count; j = j + 1d){
						afterPoint.numberArray[(int)(j)] = nGetNumberFromNumberCharacterForBase(n[(int)(i + j)], basex);
					}
					i = i + count;
					if(i < n.Length){
						success = nExtractPartsFromNumberStringFromExponent(n, basex, i, exponentIsPositive, exponent, errorMessages);
					}else{
						exponent.numberArray = new double [0];
						success = true;
					}
				}else{
					success = false;
					errorMessages.stringx = "There must be at least one digit after the decimal point.".ToCharArray();
				}
			}else{
				success = false;
				errorMessages.stringx = "There must be at least one digit after the decimal point.".ToCharArray();
			}
		}else if(basex <= 14d && (n[(int)(i)] == 'e' || n[(int)(i)] == 'E')){
			if(i < n.Length){
				success = nExtractPartsFromNumberStringFromExponent(n, basex, i, exponentIsPositive, exponent, errorMessages);
				afterPoint.numberArray = new double [0];
			}else{
				success = false;
				errorMessages.stringx = "There must be at least one digit after the exponent.".ToCharArray();
			}
		}else{
			success = false;
			errorMessages.stringx = "Expected decimal point or exponent symbol.".ToCharArray();
		}

		return success;
	}


	public static bool nExtractPartsFromNumberStringFromExponent(char [] n, double basex, double i, BooleanReference exponentIsPositive, NumberArrayReference exponent, StringReference errorMessages){
		bool success, done;
		double count, j;
		if(basex <= 14d && (n[(int)(i)] == 'e' || n[(int)(i)] == 'E')){
			i = i + 1d;
			if(i < n.Length){
				if(n[(int)(i)] == '-'){
					exponentIsPositive.booleanValue = false;
					i = i + 1d;
				}else if(n[(int)(i)] == '+'){
					exponentIsPositive.booleanValue = true;
					i = i + 1d;
				}
				if(i < n.Length){
					done = false;
					count = 0d;
					for(; i + count < n.Length && !done; ){
						if(nCharacterIsNumberCharacterInBase(n[(int)(i + count)], basex)){
							count = count + 1d;
						}else{
							done = true;
						}
					}
					if(count >= 1d){
						exponent.numberArray = new double [(int)(count)];
						for(j = 0d; j < count; j = j + 1d){
							exponent.numberArray[(int)(j)] = nGetNumberFromNumberCharacterForBase(n[(int)(i + j)], basex);
						}
						i = i + count;
						if(i == n.Length){
							success = true;
						}else{
							success = false;
							errorMessages.stringx = "There cannot be any characters past the exponent of the number.".ToCharArray();
						}
					}else{
						success = false;
						errorMessages.stringx = "There must be at least one digit after the decimal point.".ToCharArray();
					}
				}else{
					success = false;
					errorMessages.stringx = "There must be at least one digit after the exponent symbol.".ToCharArray();
				}
			}else{
				success = false;
				errorMessages.stringx = "There must be at least one digit after the exponent symbol.".ToCharArray();
			}
		}else{
			success = false;
			errorMessages.stringx = "Expected exponent symbol.".ToCharArray();
		}

		return success;
	}


	public static double nGetNumberFromNumberCharacterForBase(char c, double basex){
		char [] numberTable;
		double i;
		double position;
		numberTable = nGetDigitCharacterTable();
		position = 0d;

		for(i = 0d; i < basex; i = i + 1d){
			if(numberTable[(int)(i)] == c){
				position = i;
			}
		}

		return position;
	}


	public static bool nCharacterIsNumberCharacterInBase(char c, double basex){
		char [] numberTable;
		double i;
		bool found;
		numberTable = nGetDigitCharacterTable();
		found = false;

		for(i = 0d; i < basex; i = i + 1d){
			if(numberTable[(int)(i)] == c){
				found = true;
			}
		}

		return found;
	}


	public static double [] nStringToNumberArray(char [] str){
		NumberArrayReference numberArrayReference;
		StringReference stringReference;
		double [] numbers;
		numberArrayReference = new NumberArrayReference();
		stringReference = new StringReference();

		nStringToNumberArrayWithCheck(str, numberArrayReference, stringReference);

		numbers = numberArrayReference.numberArray;

		delete(numberArrayReference);
		delete(stringReference);

		return numbers;
	}


	public static bool nStringToNumberArrayWithCheck(char [] str, NumberArrayReference numberArrayReference, StringReference errorMessage){
		StringReference [] numberStrings;
		double [] numbers;
		double i;
		char [] numberString, trimmedNumberString;
		bool success;
		NumberReference numberReference;
		numberStrings = strSplitByString(str, ",".ToCharArray());

		numbers = new double [(int)(numberStrings.Length)];
		success = true;
		numberReference = new NumberReference();

		for(i = 0d; i < numberStrings.Length; i = i + 1d){
			numberString = numberStrings[(int)(i)].stringx;
			trimmedNumberString = strTrim(numberString);
			success = nCreateNumberFromDecimalStringWithCheck(trimmedNumberString, numberReference, errorMessage);
			numbers[(int)(i)] = numberReference.numberValue;
			FreeStringReference(numberStrings[(int)(i)]);
			delete(trimmedNumberString);
		}

		delete(numberStrings);
		delete(numberReference);

		numberArrayReference.numberArray = numbers;

		return success;
	}


	public static double [] AddNumber(double [] list, double a){
		double [] newlist;
		double i;
		newlist = new double [(int)(list.Length + 1d)];
		for(i = 0d; i < list.Length; i = i + 1d){
			newlist[(int)(i)] = list[(int)(i)];
		}
		newlist[(int)(list.Length)] = a;
		
		delete(list);
		
		return newlist;
	}


	public static void AddNumberRef(NumberArrayReference list, double i){
		list.numberArray = AddNumber(list.numberArray, i);
	}


	public static double [] RemoveNumber(double [] list, double n){
		double [] newlist;
		double i;
		newlist = new double [(int)(list.Length - 1d)];

		for(i = 0d; i < list.Length; i = i + 1d){
			if(i < n){
				newlist[(int)(i)] = list[(int)(i)];
			}
			if(i > n){
				newlist[(int)(i - 1d)] = list[(int)(i)];
			}
		}
		
		delete(list);
		
		return newlist;
	}


	public static double GetNumberRef(NumberArrayReference list, double i){
		return list.numberArray[(int)(i)];
	}


	public static void RemoveNumberRef(NumberArrayReference list, double i){
		list.numberArray = RemoveNumber(list.numberArray, i);
	}


	public static StringReference [] AddString(StringReference [] list, StringReference a){
		StringReference [] newlist;
		double i;
		newlist = new StringReference [(int)(list.Length + 1d)];

		for(i = 0d; i < list.Length; i = i + 1d){
			newlist[(int)(i)] = list[(int)(i)];
		}
		newlist[(int)(list.Length)] = a;
		
		delete(list);
		
		return newlist;
	}


	public static void AddStringRef(StringArrayReference list, StringReference i){
		list.stringArray = AddString(list.stringArray, i);
	}


	public static StringReference [] RemoveString(StringReference [] list, double n){
		StringReference [] newlist;
		double i;
		newlist = new StringReference [(int)(list.Length - 1d)];

		for(i = 0d; i < list.Length; i = i + 1d){
			if(i < n){
				newlist[(int)(i)] = list[(int)(i)];
			}
			if(i > n){
				newlist[(int)(i - 1d)] = list[(int)(i)];
			}
		}
		
		delete(list);
		
		return newlist;
	}


	public static StringReference GetStringRef(StringArrayReference list, double i){
		return list.stringArray[(int)(i)];
	}


	public static void RemoveStringRef(StringArrayReference list, double i){
		list.stringArray = RemoveString(list.stringArray, i);
	}


	public static bool [] AddBoolean(bool [] list, bool a){
		bool [] newlist;
		double i;
		newlist = new bool [(int)(list.Length + 1d)];
		for(i = 0d; i < list.Length; i = i + 1d){
			newlist[(int)(i)] = list[(int)(i)];
		}
		newlist[(int)(list.Length)] = a;
		
		delete(list);
		
		return newlist;
	}


	public static void AddBooleanRef(BooleanArrayReference list, bool i){
		list.booleanArray = AddBoolean(list.booleanArray, i);
	}


	public static bool [] RemoveBoolean(bool [] list, double n){
		bool [] newlist;
		double i;
		newlist = new bool [(int)(list.Length - 1d)];

		for(i = 0d; i < list.Length; i = i + 1d){
			if(i < n){
				newlist[(int)(i)] = list[(int)(i)];
			}
			if(i > n){
				newlist[(int)(i - 1d)] = list[(int)(i)];
			}
		}
		
		delete(list);
		
		return newlist;
	}


	public static bool GetBooleanRef(BooleanArrayReference list, double i){
		return list.booleanArray[(int)(i)];
	}


	public static void RemoveDecimalRef(BooleanArrayReference list, double i){
		list.booleanArray = RemoveBoolean(list.booleanArray, i);
	}


	public static char [] AddCharacter(char [] list, char a){
		char [] newlist;
		double i;
		newlist = new char [(int)(list.Length + 1d)];
		for(i = 0d; i < list.Length; i = i + 1d){
			newlist[(int)(i)] = list[(int)(i)];
		}
		newlist[(int)(list.Length)] = a;
		
		delete(list);
		
		return newlist;
	}


	public static void AddCharacterRef(StringReference list, char i){
		list.stringx = AddCharacter(list.stringx, i);
	}


	public static char [] RemoveCharacter(char [] list, double n){
		char [] newlist;
		double i;
		newlist = new char [(int)(list.Length - 1d)];

		for(i = 0d; i < list.Length; i = i + 1d){
			if(i < n){
				newlist[(int)(i)] = list[(int)(i)];
			}
			if(i > n){
				newlist[(int)(i - 1d)] = list[(int)(i)];
			}
		}

		delete(list);

		return newlist;
	}


	public static char GetCharacterRef(StringReference list, double i){
		return list.stringx[(int)(i)];
	}


	public static void RemoveCharacterRef(StringReference list, double i){
		list.stringx = RemoveCharacter(list.stringx, i);
	}


	public static double [] StringToNumberArray(char [] stringx){
		double i;
		double [] array;
		array = new double [(int)(stringx.Length)];

		for(i = 0d; i < stringx.Length; i = i + 1d){
			array[(int)(i)] = stringx[(int)(i)];
		}
		return array;
	}


	public static char [] NumberArrayToString(double [] array){
		double i;
		char [] stringx;
		stringx = new char [(int)(array.Length)];

		for(i = 0d; i < array.Length; i = i + 1d){
			stringx[(int)(i)] = (char)(array[(int)(i)]);
		}
		return stringx;
	}


	public static bool NumberArraysEqual(double [] a, double [] b){
		bool equal;
		double i;
		equal = true;
		if(a.Length == b.Length){
			for(i = 0d; i < a.Length && equal; i = i + 1d){
				if(a[(int)(i)] != b[(int)(i)]){
					equal = false;
				}
			}
		}else{
			equal = false;
		}

		return equal;
	}


	public static bool BooleanArraysEqual(bool [] a, bool [] b){
		bool equal;
		double i;
		equal = true;
		if(a.Length == b.Length){
			for(i = 0d; i < a.Length && equal; i = i + 1d){
				if(a[(int)(i)] != b[(int)(i)]){
					equal = false;
				}
			}
		}else{
			equal = false;
		}

		return equal;
	}


	public static bool StringsEqual(char [] a, char [] b){
		bool equal;
		double i;
		equal = true;
		if(a.Length == b.Length){
			for(i = 0d; i < a.Length && equal; i = i + 1d){
				if(a[(int)(i)] != b[(int)(i)]){
					equal = false;
				}
			}
		}else{
			equal = false;
		}

		return equal;
	}


	public static void FillNumberArray(double [] a, double value){
		double i;
		for(i = 0d; i < a.Length; i = i + 1d){
			a[(int)(i)] = value;
		}
	}


	public static void FillString(char [] a, char value){
		double i;
		for(i = 0d; i < a.Length; i = i + 1d){
			a[(int)(i)] = value;
		}
	}


	public static void FillBooleanArray(bool [] a, bool value){
		double i;
		for(i = 0d; i < a.Length; i = i + 1d){
			a[(int)(i)] = value;
		}
	}


	public static bool FillNumberArrayInterval(double [] a, double value, double from, double to){
		double i;
		bool success;
		if(from >= 0d && from < a.Length && to >= 0d && to < a.Length){
			for(i = from; i < to; i = i + 1d){
				a[(int)(i)] = value;
			}
			success = true;
		}else{
			success = false;
		}

		return success;
	}


	public static bool FillBooleanArrayInterval(bool [] a, bool value, double from, double to){
		double i;
		bool success;
		if(from >= 0d && from < a.Length && to >= 0d && to < a.Length){
			for(i = Max(from, 0d); i < Min(to, a.Length); i = i + 1d){
				a[(int)(i)] = value;
			}
			success = true;
		}else{
			success = false;
		}

		return success;
	}


	public static bool FillStringInterval(char [] a, char value, double from, double to){
		double i;
		bool success;
		if(from >= 0d && from < a.Length && to >= 0d && to < a.Length){
			for(i = Max(from, 0d); i < Min(to, a.Length); i = i + 1d){
				a[(int)(i)] = value;
			}
			success = true;
		}else{
			success = false;
		}

		return success;
	}


	public static double [] CopyNumberArray(double [] a){
		double i;
		double [] n;
		n = new double [(int)(a.Length)];

		for(i = 0d; i < a.Length; i = i + 1d){
			n[(int)(i)] = a[(int)(i)];
		}

		return n;
	}


	public static bool [] CopyBooleanArray(bool [] a){
		double i;
		bool [] n;
		n = new bool [(int)(a.Length)];

		for(i = 0d; i < a.Length; i = i + 1d){
			n[(int)(i)] = a[(int)(i)];
		}

		return n;
	}


	public static char [] CopyString(char [] a){
		double i;
		char [] n;
		n = new char [(int)(a.Length)];

		for(i = 0d; i < a.Length; i = i + 1d){
			n[(int)(i)] = a[(int)(i)];
		}

		return n;
	}


	public static bool CopyNumberArrayRange(double [] a, double from, double to, NumberArrayReference copyReference){
		double i, length;
		double [] n;
		bool success;
		if(from >= 0d && from < a.Length && to >= 0d && to < a.Length && from <= to){
			length = to - from;
			n = new double [(int)(length)];
			for(i = 0d; i < length; i = i + 1d){
				n[(int)(i)] = a[(int)(from + i)];
			}
			copyReference.numberArray = n;
			success = true;
		}else{
			success = false;
		}

		return success;
	}


	public static bool CopyBooleanArrayRange(bool [] a, double from, double to, BooleanArrayReference copyReference){
		double i, length;
		bool [] n;
		bool success;
		if(from >= 0d && from < a.Length && to >= 0d && to < a.Length && from <= to){
			length = to - from;
			n = new bool [(int)(length)];
			for(i = 0d; i < length; i = i + 1d){
				n[(int)(i)] = a[(int)(from + i)];
			}
			copyReference.booleanArray = n;
			success = true;
		}else{
			success = false;
		}

		return success;
	}


	public static bool CopyStringRange(char [] a, double from, double to, StringReference copyReference){
		double i, length;
		char [] n;
		bool success;
		if(from >= 0d && from < a.Length && to >= 0d && to < a.Length && from <= to){
			length = to - from;
			n = new char [(int)(length)];
			for(i = 0d; i < length; i = i + 1d){
				n[(int)(i)] = a[(int)(from + i)];
			}
			copyReference.stringx = n;
			success = true;
		}else{
			success = false;
		}

		return success;
	}


	public static bool IsLastElement(double length, double index){
		return index + 1d == length;
	}


	public static double [] CreateNumberArray(double length, double value){
		double [] array;
		array = new double [(int)(length)];
		FillNumberArray(array, value);

		return array;
	}


	public static bool [] CreateBooleanArray(double length, bool value){
		bool [] array;
		array = new bool [(int)(length)];
		FillBooleanArray(array, value);

		return array;
	}


	public static char [] CreateString(double length, char value){
		char [] array;
		array = new char [(int)(length)];
		FillString(array, value);

		return array;
	}


	public static void AssertFalse(bool b, NumberReference failures){
		if(b){
			failures.numberValue = failures.numberValue + 1d;
		}
	}


	public static void AssertTrue(bool b, NumberReference failures){
		if(!b){
			failures.numberValue = failures.numberValue + 1d;
		}
	}


	public static void AssertEquals(double a, double b, NumberReference failures){
		if(a != b){
			failures.numberValue = failures.numberValue + 1d;
		}
	}


	public static void AssertBooleansEqual(bool a, bool b, NumberReference failures){
		if(a != b){
			failures.numberValue = failures.numberValue + 1d;
		}
	}


	public static void AssertCharactersEqual(char a, char b, NumberReference failures){
		if(a != b){
			failures.numberValue = failures.numberValue + 1d;
		}
	}


	public static void AssertStringEquals(char [] a, char [] b, NumberReference failures){
		if(!StringsEqual(a, b)){
			failures.numberValue = failures.numberValue + 1d;
		}
	}


	public static void AssertNumberArraysEqual(double [] a, double [] b, NumberReference failures){
		double i;
		if(a.Length == b.Length){
			for(i = 0d; i < a.Length; i = i + 1d){
				AssertEquals(a[(int)(i)], b[(int)(i)], failures);
			}
		}else{
			failures.numberValue = failures.numberValue + 1d;
		}
	}


	public static void AssertBooleanArraysEqual(bool [] a, bool [] b, NumberReference failures){
		double i;
		if(a.Length == b.Length){
			for(i = 0d; i < a.Length; i = i + 1d){
				AssertBooleansEqual(a[(int)(i)], b[(int)(i)], failures);
			}
		}else{
			failures.numberValue = failures.numberValue + 1d;
		}
	}


	public static void AssertStringArraysEqual(StringReference [] a, StringReference [] b, NumberReference failures){
		double i;
		if(a.Length == b.Length){
			for(i = 0d; i < a.Length; i = i + 1d){
				AssertStringEquals(a[(int)(i)].stringx, b[(int)(i)].stringx, failures);
			}
		}else{
			failures.numberValue = failures.numberValue + 1d;
		}
	}


	public static char charToLowerCase(char character){
		char toReturn;
		toReturn = character;
		if(character == 'A'){
			toReturn = 'a';
		}else if(character == 'B'){
			toReturn = 'b';
		}else if(character == 'C'){
			toReturn = 'c';
		}else if(character == 'D'){
			toReturn = 'd';
		}else if(character == 'E'){
			toReturn = 'e';
		}else if(character == 'F'){
			toReturn = 'f';
		}else if(character == 'G'){
			toReturn = 'g';
		}else if(character == 'H'){
			toReturn = 'h';
		}else if(character == 'I'){
			toReturn = 'i';
		}else if(character == 'J'){
			toReturn = 'j';
		}else if(character == 'K'){
			toReturn = 'k';
		}else if(character == 'L'){
			toReturn = 'l';
		}else if(character == 'M'){
			toReturn = 'm';
		}else if(character == 'N'){
			toReturn = 'n';
		}else if(character == 'O'){
			toReturn = 'o';
		}else if(character == 'P'){
			toReturn = 'p';
		}else if(character == 'Q'){
			toReturn = 'q';
		}else if(character == 'R'){
			toReturn = 'r';
		}else if(character == 'S'){
			toReturn = 's';
		}else if(character == 'T'){
			toReturn = 't';
		}else if(character == 'U'){
			toReturn = 'u';
		}else if(character == 'V'){
			toReturn = 'v';
		}else if(character == 'W'){
			toReturn = 'w';
		}else if(character == 'X'){
			toReturn = 'x';
		}else if(character == 'Y'){
			toReturn = 'y';
		}else if(character == 'Z'){
			toReturn = 'z';
		}

		return toReturn;
	}


	public static char charToUpperCase(char character){
		char toReturn;
		toReturn = character;
		if(character == 'a'){
			toReturn = 'A';
		}else if(character == 'b'){
			toReturn = 'B';
		}else if(character == 'c'){
			toReturn = 'C';
		}else if(character == 'd'){
			toReturn = 'D';
		}else if(character == 'e'){
			toReturn = 'E';
		}else if(character == 'f'){
			toReturn = 'F';
		}else if(character == 'g'){
			toReturn = 'G';
		}else if(character == 'h'){
			toReturn = 'H';
		}else if(character == 'i'){
			toReturn = 'I';
		}else if(character == 'j'){
			toReturn = 'J';
		}else if(character == 'k'){
			toReturn = 'K';
		}else if(character == 'l'){
			toReturn = 'L';
		}else if(character == 'm'){
			toReturn = 'M';
		}else if(character == 'n'){
			toReturn = 'N';
		}else if(character == 'o'){
			toReturn = 'O';
		}else if(character == 'p'){
			toReturn = 'P';
		}else if(character == 'q'){
			toReturn = 'Q';
		}else if(character == 'r'){
			toReturn = 'R';
		}else if(character == 's'){
			toReturn = 'S';
		}else if(character == 't'){
			toReturn = 'T';
		}else if(character == 'u'){
			toReturn = 'U';
		}else if(character == 'v'){
			toReturn = 'V';
		}else if(character == 'w'){
			toReturn = 'W';
		}else if(character == 'x'){
			toReturn = 'X';
		}else if(character == 'y'){
			toReturn = 'Y';
		}else if(character == 'z'){
			toReturn = 'Z';
		}

		return toReturn;
	}


	public static bool charIsUpperCase(char character){
		bool isUpper;
		isUpper = false;
		if(character == 'A'){
			isUpper = true;
		}else if(character == 'B'){
			isUpper = true;
		}else if(character == 'C'){
			isUpper = true;
		}else if(character == 'D'){
			isUpper = true;
		}else if(character == 'E'){
			isUpper = true;
		}else if(character == 'F'){
			isUpper = true;
		}else if(character == 'G'){
			isUpper = true;
		}else if(character == 'H'){
			isUpper = true;
		}else if(character == 'I'){
			isUpper = true;
		}else if(character == 'J'){
			isUpper = true;
		}else if(character == 'K'){
			isUpper = true;
		}else if(character == 'L'){
			isUpper = true;
		}else if(character == 'M'){
			isUpper = true;
		}else if(character == 'N'){
			isUpper = true;
		}else if(character == 'O'){
			isUpper = true;
		}else if(character == 'P'){
			isUpper = true;
		}else if(character == 'Q'){
			isUpper = true;
		}else if(character == 'R'){
			isUpper = true;
		}else if(character == 'S'){
			isUpper = true;
		}else if(character == 'T'){
			isUpper = true;
		}else if(character == 'U'){
			isUpper = true;
		}else if(character == 'V'){
			isUpper = true;
		}else if(character == 'W'){
			isUpper = true;
		}else if(character == 'X'){
			isUpper = true;
		}else if(character == 'Y'){
			isUpper = true;
		}else if(character == 'Z'){
			isUpper = true;
		}

		return isUpper;
	}


	public static bool charIsLowerCase(char character){
		bool isLower;
		isLower = false;
		if(character == 'a'){
			isLower = true;
		}else if(character == 'b'){
			isLower = true;
		}else if(character == 'c'){
			isLower = true;
		}else if(character == 'd'){
			isLower = true;
		}else if(character == 'e'){
			isLower = true;
		}else if(character == 'f'){
			isLower = true;
		}else if(character == 'g'){
			isLower = true;
		}else if(character == 'h'){
			isLower = true;
		}else if(character == 'i'){
			isLower = true;
		}else if(character == 'j'){
			isLower = true;
		}else if(character == 'k'){
			isLower = true;
		}else if(character == 'l'){
			isLower = true;
		}else if(character == 'm'){
			isLower = true;
		}else if(character == 'n'){
			isLower = true;
		}else if(character == 'o'){
			isLower = true;
		}else if(character == 'p'){
			isLower = true;
		}else if(character == 'q'){
			isLower = true;
		}else if(character == 'r'){
			isLower = true;
		}else if(character == 's'){
			isLower = true;
		}else if(character == 't'){
			isLower = true;
		}else if(character == 'u'){
			isLower = true;
		}else if(character == 'v'){
			isLower = true;
		}else if(character == 'w'){
			isLower = true;
		}else if(character == 'x'){
			isLower = true;
		}else if(character == 'y'){
			isLower = true;
		}else if(character == 'z'){
			isLower = true;
		}

		return isLower;
	}


	public static bool charIsLetter(char character){
		return charIsUpperCase(character) || charIsLowerCase(character);
	}


	public static bool charIsNumber(char character){
		bool isNumber;
		isNumber = false;
		if(character == '0'){
			isNumber = true;
		}else if(character == '1'){
			isNumber = true;
		}else if(character == '2'){
			isNumber = true;
		}else if(character == '3'){
			isNumber = true;
		}else if(character == '4'){
			isNumber = true;
		}else if(character == '5'){
			isNumber = true;
		}else if(character == '6'){
			isNumber = true;
		}else if(character == '7'){
			isNumber = true;
		}else if(character == '8'){
			isNumber = true;
		}else if(character == '9'){
			isNumber = true;
		}

		return isNumber;
	}


	public static bool charIsWhiteSpace(char character){
		bool isWhiteSpace;
		isWhiteSpace = false;
		if(character == ' '){
			isWhiteSpace = true;
		}else if(character == '\t'){
			isWhiteSpace = true;
		}else if(character == '\n'){
			isWhiteSpace = true;
		}else if(character == '\r'){
			isWhiteSpace = true;
		}

		return isWhiteSpace;
	}


	public static bool charIsSymbol(char character){
		bool isSymbol;
		isSymbol = false;
		if(character == '!'){
			isSymbol = true;
		}else if(character == '\"'){
			isSymbol = true;
		}else if(character == '#'){
			isSymbol = true;
		}else if(character == '$'){
			isSymbol = true;
		}else if(character == '%'){
			isSymbol = true;
		}else if(character == '&'){
			isSymbol = true;
		}else if(character == '\''){
			isSymbol = true;
		}else if(character == '('){
			isSymbol = true;
		}else if(character == ')'){
			isSymbol = true;
		}else if(character == '*'){
			isSymbol = true;
		}else if(character == '+'){
			isSymbol = true;
		}else if(character == ','){
			isSymbol = true;
		}else if(character == '-'){
			isSymbol = true;
		}else if(character == '.'){
			isSymbol = true;
		}else if(character == '/'){
			isSymbol = true;
		}else if(character == ':'){
			isSymbol = true;
		}else if(character == ';'){
			isSymbol = true;
		}else if(character == '<'){
			isSymbol = true;
		}else if(character == '='){
			isSymbol = true;
		}else if(character == '>'){
			isSymbol = true;
		}else if(character == '?'){
			isSymbol = true;
		}else if(character == '@'){
			isSymbol = true;
		}else if(character == '['){
			isSymbol = true;
		}else if(character == '\\'){
			isSymbol = true;
		}else if(character == ']'){
			isSymbol = true;
		}else if(character == '^'){
			isSymbol = true;
		}else if(character == '_'){
			isSymbol = true;
		}else if(character == '`'){
			isSymbol = true;
		}else if(character == '{'){
			isSymbol = true;
		}else if(character == '|'){
			isSymbol = true;
		}else if(character == '}'){
			isSymbol = true;
		}else if(character == '~'){
			isSymbol = true;
		}

		return isSymbol;
	}


	public static double Negate(double x){
		return -x;
	}


	public static double Positive(double x){
		return +x;
	}


	public static double Factorial(double x){
		double i, f;
		f = 1d;

		for(i = 2d; i <= x; i = i + 1d){
			f = f*i;
		}

		return f;
	}


	public static double Roundx(double x){
		return Floor(x + 0.5);
	}


	public static double BankersRound(double x){
		double r;
		if(Absolute(x - Truncate(x)) == 0.5){
			if(!DivisibleBy(Roundx(x), 2d)){
				r = Roundx(x) - 1d;
			}else{
				r = Roundx(x);
			}
		}else{
			r = Roundx(x);
		}

		return r;
	}


	public static double Ceil(double x){
		return Ceiling(x);
	}


	public static double Floorx(double x){
		return Floor(x);
	}


	public static double Truncate(double x){
		double t;
		if(x >= 0d){
			t = Floor(x);
		}else{
			t = Ceiling(x);
		}

		return t;
	}


	public static double Absolute(double x){
		return Abs(x);
	}


	public static double Logarithm(double x){
		return Log10(x);
	}


	public static double NaturalLogarithm(double x){
		return Log(x);
	}


	public static double Sinx(double x){
		return Sin(x);
	}


	public static double Cosx(double x){
		return Cos(x);
	}


	public static double Tanx(double x){
		return Tan(x);
	}


	public static double Asinx(double x){
		return Asin(x);
	}


	public static double Acosx(double x){
		return Acos(x);
	}


	public static double Atanx(double x){
		return Atan(x);
	}


	public static double Atan2(double y, double x){
		double a;
		a = 0d;

		if(x > 0d){
			a = Atanx(y/x);
		}else if(x < 0d && y >= 0d){
			a = Atanx(y/x) + PI;
		}else if(x < 0d && y < 0d){
			a = Atanx(y/x) - PI;
		}else if(x == 0d && y > 0d){
			a = PI/2d;
		}else if(x == 0d && y < 0d){
			a = -PI/2d;
		}

		return a;
	}


	public static double Squareroot(double x){
		return Sqrt(x);
	}


	public static double Expx(double x){
		return Exp(x);
	}


	public static bool DivisibleBy(double a, double b){
		return ((a%b) == 0d);
	}


	public static double Combinations(double n, double k){
		return Factorial(n)/(Factorial(n - k)*Factorial(k));
	}


	public static bool EpsilonCompareApproximateDigits(double a, double b, double digits){
		double ad, bd, d, epsilon;
		bool ret;
		if(a < 0d && b < 0d || a > 0d && b > 0d){
			if(a < 0d && b < 0d){
				a = -a;
				b = -b;
			}
			ad = Log10(a);
			bd = Log10(b);
			d = Max(ad, bd);
			epsilon = Pow(10d, d - digits);
			ret = Abs(a - b) > epsilon;
		}else{
			ret = false;
		}

		return ret;
	}


	public static bool EpsilonCompare(double a, double b, double epsilon){
		return Abs(a - b) < epsilon;
	}


	public static double GreatestCommonDivisor(double a, double b){
		double t;
		for(; b != 0d; ){
			t = b;
			b = a%b;
			a = t;
		}

		return a;
	}


	public static bool IsInteger(double a){
		return (a - Floor(a)) == 0d;
	}


	public static bool GreatestCommonDivisorWithCheck(double a, double b, NumberReference gcdReference){
		bool success;
		double gcd;
		if(IsInteger(a) && IsInteger(b)){
			gcd = GreatestCommonDivisor(a, b);
			gcdReference.numberValue = gcd;
			success = true;
		}else{
			success = false;
		}

		return success;
	}


	public static double LeastCommonMultiple(double a, double b){
		double lcm;
		if(a > 0d && b > 0d){
			lcm = Abs(a*b)/GreatestCommonDivisor(a, b);
		}else{
			lcm = 0d;
		}

		return lcm;
	}


	public static double Sign(double a){
		double s;
		if(a > 0d){
			s = 1d;
		}else if(a < 0d){
			s = -1d;
		}else{
			s = 0d;
		}

		return s;
	}


	public static double Maxx(double a, double b){
		return Max(a, b);
	}


	public static double Minx(double a, double b){
		return Min(a, b);
	}


	public static double Power(double a, double b){
		return Pow(a, b);
	}


	public static void delete(System.Object objectx){
		// C# has garbage collection.
	}
}

}